dnl----------------------------------------------------------------------------
dnl Locate Bison++ installation
dnl----------------------------------------------------------------------------
dnl
AC_DEFUN([MN_BISON_PLUS_PLUS_CHECK],
 [ bison_plusplus_path=""
   AC_ARG_WITH(bisonpp,
     [  --with-bisonpp=<path>   specify the directory of bison++],
     [ bison_plusplus_path="$with_bisonpp"
     ])
   mn_save_path=$PATH
   PATH="$PATH:$bison_plusplus_path"
   AC_PATH_PROG(BISONXX,bison++,no)
   PATH="$mn_save_path"
 ])
 AC_DEFUN([MN_BISON_PLUS_PLUS_EVAL],
 [ if test "x$BISONXX" = "xno"; then
     AC_MSG_WARN([Bison++ is required to (re)build CINSim's formula parser.])
     mn_all_tests=bad
   fi
 ])


dnl----------------------------------------------------------------------------
dnl Locate the XSD-grammar file of the GUI
dnl----------------------------------------------------------------------------
dnl
AC_DEFUN([CINSIM_GUI_CHECK],
 [ GUI_PATH="/usr/local/cinsim-gui"
   AC_ARG_WITH(gui,
     [  --with-gui=<path>   specify the directory of the cinsim gui],
     [ GUI_PATH="$with_gui"
     ])
   GRAMAR="$GUI_PATH/etc/schemas/CINSimPlus.xsd"
 ])


dnl----------------------------------------------------------------------------
dnl Check for existing XSD grammar file of the GUI (required)
dnl----------------------------------------------------------------------------
dnl
AC_DEFUN([CINSIM_GUI_EVAL],
 [ AC_MSG_CHECKING([for $GRAMAR])
   if ! test -f "$GRAMAR" ; then
     echo no
     AC_MSG_ERROR([Failed to locate the XSD gramar file of the GUI!])
     GUI_PATH=""
     gui_all_tests=bad
   else
	echo yes
   fi
   AC_SUBST(GUI_PATH)
 ])


dnl ---------------------------------------------------------------------------
dnl Find a file (or one of more files in a list of dirs)
dnl ---------------------------------------------------------------------------
dnl
AC_DEFUN([AC_FIND_FILE],
[
for i in $2;
do
  for j in $1;
  do
    echo "configure: __oline__: $i/$j" >&AC_FD_CC
    if test -r "$i/$j"; then
      echo "taking that" >&AC_FD_CC
      break 2
    fi
  done
done
])

dnl ---------------------------------------------------------------------------
dnl  xerces-c
dnl ---------------------------------------------------------------------------

AC_DEFUN([ACX_XERCES], [

AC_ARG_WITH(xerces-prefix, [  --with-xerces-prefix=PFX   Prefix where Xerces-C is installed (optional)],[xerces_prefix="$withval"], [xerces_prefix=""])

AC_LANG_PUSH([C++])
AC_MSG_CHECKING([for Xerces-C])

if test -n "$xerces_prefix" ; then
  CXXFLAGS="$CXXFLAGS -I$xerces_prefix/include"
  LDFLAGS="$LDFLAGS -L$xerces_prefix/lib"
fi

save_libs="$LIBS"

XERCESLIBS="-lxerces-c"

LIBS="$XERCESLIBS $LIBS"

AC_LINK_IFELSE([
 AC_LANG_PROGRAM(
  [
  #include <xercesc/parsers/XercesDOMParser.hpp>
  #include <iostream>
  XERCES_CPP_NAMESPACE_USE
  ],
  [
  XercesDOMParser* parser = new XercesDOMParser();
  ])],
 [AC_MSG_RESULT([yes])],
 [failure="yes"
])

if test -n "$failure" ; then
 AC_MSG_RESULT([no])
 AC_MSG_ERROR([Xerces-C library and development files required!])
fi
 
LIBS="$save_libs"

AC_SUBST(XERCESLIBS)

AC_LANG_POP
])

dnl ---------------------------------------------------------------------------
dnl GNU Scientific Library
dnl ---------------------------------------------------------------------------
dnl
AC_DEFUN([ACX_GSL], [

AC_ARG_WITH(gsl-prefix, [  --with-gsl-prefix=PFX   Prefix where GSL is installed (optional)],[gsl_prefix="$withval"], [gsl_prefix=""])

AC_LANG_PUSH([C++])
AC_MSG_CHECKING([for GSL])

if test -n "$gsl_prefix" ; then
  CXXFLAGS="$CXXFLAGS -I$gsl_prefix/include"
  LDFLAGS="$LDFLAGS -L$gsl_prefix/lib"
fi

save_libs="$LIBS"

GSLLIBS="-lm -lgsl -lgslcblas"
LIBS="$GSLLIBS $LIBS"

AC_LINK_IFELSE([
 AC_LANG_PROGRAM(
  [
  #include<gsl/gsl_math.h>  
  #include<gsl/gsl_linalg.h>
  ],
  [
  gsl_matrix *X = gsl_matrix_alloc(2, 2);
  gsl_matrix_free(X);
  ])],
 [AC_MSG_RESULT([yes])],
 [failure="yes"
])

if test -n "$failure" ; then
 AC_MSG_RESULT([no])
 AC_MSG_ERROR([Gnu scientific library and development files required!])
fi

LIBS="$save_libs"

AC_SUBST(GSLLIBS)

AC_LANG_POP
])

dnl @synopsis ACX_MPI([ACTION-IF-FOUND[, ACTION-IF-NOT-FOUND]])
dnl
dnl @summary figure out how to compile/link code with MPI
dnl
dnl This macro tries to find out how to compile programs that use MPI
dnl (Message Passing Interface), a standard API for parallel process
dnl communication (see http://www-unix.mcs.anl.gov/mpi/)
dnl
dnl On success, it sets the MPICC, MPICXX, or MPIF77 output variable to
dnl the name of the MPI compiler, depending upon the current language.
dnl (This may just be $CC/$CXX/$F77, but is more often something like
dnl mpicc/mpiCC/mpif77.) It also sets MPILIBS to any libraries that are
dnl needed for linking MPI (e.g. -lmpi, if a special
dnl MPICC/MPICXX/MPIF77 was not found).
dnl
dnl If you want to compile everything with MPI, you should set:
dnl
dnl     CC="$MPICC" #OR# CXX="$MPICXX" #OR# F77="$MPIF77"
dnl     LIBS="$MPILIBS $LIBS"
dnl
dnl NOTE: The above assumes that you will use $CC (or whatever) for
dnl linking as well as for compiling. (This is the default for automake
dnl and most Makefiles.)
dnl
dnl The user can force a particular library/compiler by setting the
dnl MPICC/MPICXX/MPIF77 and/or MPILIBS environment variables.
dnl
dnl ACTION-IF-FOUND is a list of shell commands to run if an MPI
dnl library is found, and ACTION-IF-NOT-FOUND is a list of commands to
dnl run it if it is not found. If ACTION-IF-FOUND is not specified, the
dnl default action will define HAVE_MPI.
dnl
dnl @category InstalledPackages
dnl @author Steven G. Johnson <stevenj@alum.mit.edu>
dnl @version 2005-09-02
dnl @license GPLWithACException

AC_DEFUN([ACX_MPI], [
AC_PREREQ(2.50) dnl for AC_LANG_CASE

AC_LANG_CASE([C], [
	AC_REQUIRE([AC_PROG_CC])
	AC_ARG_VAR(MPICC,[MPI C compiler command])
	AC_CHECK_PROGS(MPICC, mpicc hcc mpcc mpcc_r mpxlc cmpicc, $CC)
	acx_mpi_save_CC="$CC"
	CC="$MPICC"
	AC_SUBST(MPICC)
	AC_SUBST(CC_DEFAULT,"$acx_mpi_save_CC")
],
[C++], [
	AC_REQUIRE([AC_PROG_CXX])
	AC_ARG_VAR(MPICXX,[MPI C++ compiler command])
	AC_CHECK_PROGS(MPICXX, mpic++ mpiCC mpicxx mpCC hcp mpxlC mpxlC_r cmpic++, $CXX)
	acx_mpi_save_CXX="$CXX"
	CXX="$MPICXX"
	AC_SUBST(MPICXX)
	AC_SUBST(CXX_DEFAULT,"$acx_mpi_save_CXX")
],
[Fortran 77], [
	AC_REQUIRE([AC_PROG_F77])
	AC_ARG_VAR(MPIF77,[MPI Fortran compiler command])
	AC_CHECK_PROGS(MPIF77, mpif77 hf77 mpxlf mpf77 mpif90 mpf90 mpxlf90 mpxlf95 mpxlf_r cmpifc cmpif90c, $F77)
	acx_mpi_save_F77="$F77"
	F77="$MPIF77"
	AC_SUBST(MPIF77)
])

if test x = x"$MPILIBS"; then
	AC_LANG_CASE([C], [AC_CHECK_FUNC(MPI_Init, [MPILIBS=" "])],
		[C++], [AC_CHECK_FUNC(MPI_Init, [MPILIBS=" "])],
		[Fortran 77], [AC_MSG_CHECKING([for MPI_Init])
			AC_TRY_LINK([],[      call MPI_Init], [MPILIBS=" "
				AC_MSG_RESULT(yes)], [AC_MSG_RESULT(no)])])
fi
if test x = x"$MPILIBS"; then
	AC_CHECK_LIB(mpi, MPI_Init, [MPILIBS="-lmpi"])
fi
if test x = x"$MPILIBS"; then
	AC_CHECK_LIB(mpich, MPI_Init, [MPILIBS="-lmpich"])
fi

dnl We have to use AC_TRY_COMPILE and not AC_CHECK_HEADER because the
dnl latter uses $CPP, not $CC (which may be mpicc).
AC_LANG_CASE([C], [if test x != x"$MPILIBS"; then
	AC_MSG_CHECKING([for mpi.h])
	AC_TRY_COMPILE([#include <mpi.h>],[],[AC_MSG_RESULT(yes)], [MPILIBS=""
		AC_MSG_RESULT(no)])
fi],
[C++], [if test x != x"$MPILIBS"; then
	AC_MSG_CHECKING([for mpi.h])
	AC_TRY_COMPILE([#include <mpi.h>],[],[AC_MSG_RESULT(yes)], [MPILIBS=""
		AC_MSG_RESULT(no)])
fi])

AC_LANG_CASE([C], [CC="$acx_mpi_save_CC"],
	[C++], [CXX="$acx_mpi_save_CXX"],
	[Fortran 77], [F77="$acx_mpi_save_F77"])

AC_SUBST(MPILIBS)

# Finally, execute ACTION-IF-FOUND/ACTION-IF-NOT-FOUND:
if test x = x"$MPILIBS"; then
        $2
        :
else
        ifelse([$1],,[AC_DEFINE(HAVE_MPI,1,[Define if you have the MPI library.])],[$1])
        :
fi
])dnl ACX_MPI
