#include <analyser.h>
#include <utility/xmlattribute.h>
#include <simulation.h>
#include <macros.h>

#include <cassert>

using namespace std;

Analyser::Analyser(const DOMElement* scope) : XMLElement(scope)
{
	_priority = 0;
	_firstVariable = 0;
	_lastVariable = 0;
}

bool Analyser::init(const DOMElement* scope)
{
	if(scope) setScope(scope);

	assert(Scope());

	_firstVariable = getAttribute("measureVar");
	_lastVariable = _firstVariable;
	
	if(Simulation::Instance()->Type() == Simulation::TERMINATING)
	{
		// Vaiablen beginnen bei 1!
		assert(_firstVariable>0);
			
		unsigned int ticks = (Simulation::Instance()->AnalysisEnd() - 
			Simulation::Instance()->AnalysisStart()) + 1 ;
		
		_firstVariable = (_firstVariable-1) * ticks + 1;
		_lastVariable = _firstVariable + ticks -1;
	}
	
	// Prioritaet
	_priority = getAttribute("packetPriority");

	// Name der Messung	
	_description = (string)getAttribute("description");
	
	if(_description == _UNDEFINED_)
		_description = (string)getAttribute("observationType");
	
	// Art der Auswertung
	_mtype = (string)getAttribute("measurementType");

	if(_mtype == "Quantile[p]")
		_mtype = "Q[" + (string)getAttribute("p")  + "]";
	
	// Art der Messung
	XMLAttribute obstype = getAttribute("observationType");
	if((string)obstype == "Delay") _observe = Analysable::eDelay;
	else if((string)obstype == "Latency") _observe = Analysable::eLatency;
	else if((string)obstype == "Queue") _observe = Analysable::eQueue;
	else if((string)obstype == "SourceThroughput") _observe = Analysable::eSThroughput;
	else if((string)obstype == "TargetThroughput") _observe = Analysable::eTThroughput;
	else if((string)obstype == "PriorityDelay") _observe = Analysable::ePrioriDelay;
	else if((string)obstype == "PriorityLatency") _observe = Analysable::ePrioriLatency;
	else if((string)obstype == "PriorityTargetThroughput") _observe = Analysable::ePrioriTThroughput;
	//else if((string)obstype == "PacketLosses") _observe = ePacketLoss;
	//else if((string)obstype == "Offered load") _observe = eLoad;
	else _ERROR("Unexpected type of observation (" << (string)obstype <<")!");

	return true;
}

void Analyser::observe()
{
	// transient oder stationaer?
	static bool terminating = Simulation::Instance()->Type() == Simulation::TERMINATING;
	
	// Variable fuer Messung
	unsigned int var = _firstVariable;
	
	if(terminating)
	{
		static unsigned int startTick = Simulation::Instance()->AnalysisStart();
		static unsigned int endTick = Simulation::Instance()->AnalysisEnd();
			
		unsigned int tick = Tick();
		
		// check if its a measure tick
		if (startTick > tick || tick > endTick) return;
				
		var = _firstVariable + (tick - startTick);
	}
	
	// Spezifikation der Messung
	struct Analysable::Description tmp = { _observe, var, _priority };
    			
	// Komponenten anstossen
	for (unsigned long int i=0; i < size(); i++)
	{
		at(i)->takeMeasure( tmp );
	}
}
