//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "lfuarbiter.h"
#include <simulation.h>
#include <algorithm> // for_each

REGISTER_ELEMENT(LFUArbiter, "LeastFrequentlyUsed")

LFUArbiter::LFUArbiter(const DOMElement* scope): Arbiter(scope)
{
	lfuList = new std::priority_queue <std::pair<Wire *, unsigned int>,std::vector<std::pair<Wire *, unsigned int> >, CompareProperty>;
}


LFUArbiter::~LFUArbiter()
{
}


bool LFUArbiter::init(const DOMElement* scope)
{

	if (!Inputs()) {
		std::cerr << "LFUArbiter::init : Cannot init yet - No Inputs known." << std::endl;
		return false;
	} else
		for(std::vector<Wire*>::const_iterator it = Inputs()->begin(); it!=Inputs()->end(); it++)
			wonList.push_back(std::pair<Wire*, unsigned int>((*it), 0));
	
	return true;
}

const void LFUArbiter::accept(const std::vector< Wire * >& accepted_list)
{
	// increase top element for wonlist
	++(wonList[accepted_list.front()->getInIndex()].second);
}

void LFUArbiter::schedule(std::vector< Wire * > ins)
{
	if (!lfuList->empty()) {
		std::cerr << "WARNING (NOP) : LFUArbiter::schedule : my internal list of schedule objects was not empty." << std::endl;
		forget();
	}

	// put matching in prio_queue
	for (std::vector<Wire*>::iterator it = ins.begin(); it!=ins.end(); it++)
		lfuList->push(wonList[(*it)->getInIndex()]);
}

inline Wire* LFUArbiter::pop() {
        if (lfuList->empty())
                return NULL;


	Wire* out = lfuList->top().first;
	lfuList->pop();
	return out;
}

inline const void LFUArbiter::forget() {
	delete lfuList;
	lfuList = new std::priority_queue <std::pair<Wire *, unsigned int>,std::vector<std::pair<Wire *, unsigned int> >, CompareProperty>;

}

inline Wire* LFUArbiter::front() const 
{
	if (lfuList->empty())
		return NULL;

	return lfuList->top().first;
}


