//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "mruarbiter.h"

#include <simulation.h>
#include <algorithm> // for_each

REGISTER_ELEMENT(MRUArbiter, "MostRecentlyUsed")

MRUArbiter::MRUArbiter(const DOMElement* scope): Arbiter(scope)
{
	lfuList = new std::priority_queue <std::pair<Wire *, unsigned int>,std::vector<std::pair<Wire *, unsigned int> >, CompareProperty>;
}


MRUArbiter::~MRUArbiter()
{
}


bool MRUArbiter::init(const DOMElement* scope)
{
	if (!Inputs()) {
		std::cerr << "MRUArbiter::init : Cannot init yet - No Inputs known." << std::endl;
		return false;
	} else
	{
		for(std::vector<Wire*>::const_iterator it = Inputs()->begin(); it!=Inputs()->end(); it++)
			wonList.push_back(std::pair<Wire*, unsigned int>((*it), 0));
	}
	
	return true;
}

const void MRUArbiter::accept(const std::vector< Wire * >& accepted_list)
{
	// remember the time this one has been used
	wonList[accepted_list.front()->getInIndex()].second = Tick();
}

void MRUArbiter::schedule(std::vector< Wire * > ins)
{
	if (!lfuList->empty()) {
		std::cerr << "WARNING (NOP) : MRUArbiter::schedule : my internal list of schedule objects was not empty." << std::endl;
		forget();
	}

	// put matching in prio_queue
	for (std::vector<Wire*>::iterator it = ins.begin(); it!=ins.end(); it++) {
		lfuList->push(wonList[(*it)->getInIndex()]);
	}
}

inline Wire* MRUArbiter::pop() {
	if (lfuList->empty())
		return NULL;

	Wire* out = lfuList->top().first;
	lfuList->pop();
	return out;
}

inline Wire* MRUArbiter::front() const
{
	if (lfuList->empty())
		return NULL;

	return lfuList->top().first;
}

inline const void MRUArbiter::forget() {
	delete lfuList;
	lfuList = new std::priority_queue <std::pair<Wire *, unsigned int>,std::vector<std::pair<Wire *, unsigned int> >, CompareProperty>;

}
