#include <arbitercontroller.h>
#include <packetforwarding.h>
#include <wirematrixiterator.h>
#include <arbiter.h>
#include <wire.h>
#include <flit.h>
#include <macros.h>
#include <utility/xmlattribute.h>

#include <assert.h>

#include <dlfcn.h> // dlopen-test

using namespace std;

ArbiterController::~ArbiterController()
{
	// clear the arbiter list
	while(arbiter.size()) {
		delete arbiter.back();
		arbiter.pop_back();
	};
}

bool ArbiterController::competitor(WireMatrixEntry& entry) const
{
	// Annahme: Ausgang noch nicht fertig bearbeitet
	assert(!entry.test(O_FINISHED && !entry.test(IO_WON) && !entry.test(IO_LOST)));
	// Annahme: Wenn zulaessiges Paket angezeigt, dann Flit verfuegbar
	assert(!entry.test(IO_VALID) || entry.getInWire()->getFlit(entry.getOutWire()->To()));
	
	return 
	// 1. Eingang ist zulaessig und Ausgang hat Platz
	entry.test(IO_VALID) && entry.test(IO_FIT) &&
	// 2. Flit darf bewegt werden
	entry.getInWire()->getFlit(entry.getOutWire()->To())->moveable() &&
	// 3. Ausgang deckt mindestens ein verbleibendes Ziel ab
	(*entry.remaining()) <= (*entry.targets());
}

bool ArbiterController::addArbiter() 
{
	// Annahme: XML-Element mit Informationen zu Arbiter vorhanden.
	assert(Scope());
	
	Arbiter* current = (Arbiter*)FactoryElement::Factory::self().create(getAttribute("schedulingType"));
	
	if (!current) {
		_ERROR("ArbiterController::addArbiter cannot create an Arbiter of type: " 
				<< ((std::string)getAttribute("schedulingType")));
	}
	
	arbiter.push_back(current);
	assert(_wm!=NULL);
	current->setInputs(&_wm->getInWires());
	current->init(Scope());
	current->setIndex(arbiter.size()-1);
	
	return true;
}

const Arbiter* ArbiterController::getArbiter(Wire* outwire) const
{
	// Annahme: Eintrag vorhanden
	assert(outwire->getOutIndex() < arbiter.size());

	return arbiter[outwire->getOutIndex()];;
}

bool ArbiterController::init(const DOMElement* scope) 
{
	setScope(scope);

	// nothing to do yet
	return true;
}

int ArbiterController::precondition(WireMatrixIterator* it)
{
	WireMatrixEntry& entry = it->value();
	
	// Annahme: Eingaenge zu einem Ausgang werden iteriert
	assert(it->rtti()==WireMatrixIterator::INWIRE_IT);

	return arbiter[entry.getOutWire()->getOutIndex()]->valid();
}

void ArbiterController::modify(WireMatrixIterator* it)
{
	WireMatrixEntry& entry = it->value();
	
	Wire* inwire = entry.getInWire();
	Wire* outwire = entry.getOutWire();
	Arbiter* ow_arbiter = arbiter[outwire->getOutIndex()];
	
	if(ow_arbiter->front())
	{
		// ggf. Sieger markieren
		entry.set(IO_WON, entry.test(IO_VALID) && inwire==ow_arbiter->front());
	}
	else
	{
		// kein Gewinner, also Ausgang fertig
		entry.set(O_FINISHED);
	}
	
	// Verlierer markieren
	entry.set(IO_LOST, entry.test(IO_VALID) && !entry.test(IO_WON));	
	
	// Ausgang als gescheduled markieren
	entry.set(O_SCHEDULED);
}

