#include <bufferqueue.h>
#include <registercontainer.h>
#include <buffer.h>
#include <packet.h>
#include <macros.h>
#include <simulation.h>

#include <cassert>

BufferQueue::BufferQueue(unsigned int space)
{
	maxSpace = space<1 ? 1 : space;
	rc = new RegisterContainer();
	infinite=false;
}

/*
bool BufferQueue::freeSpace(unsigned int size, unsigned int nextBuffer)
{
        if (buffer[nextBuffer]->sharedNumber == 1) {
                return (usedSpace+registerSize()+size<=maxSpace);
        } else {
		std::cerr << "shared" << std::endl;
                // das ist ein shared Buffer
                // wenn insgesamt weniger als das angeforderte belegt wird, ist es unproblematisch
                if (usedSpace+registerSize()+size <= buffer[nextBuffer]->sharedQueueMin) {
                        return true;
                } else {

                        int occupiedDistributableSpace = 0;
                        int aktBuffer = nextBuffer;
                        for (unsigned int i = 0; i < buffer[nextBuffer]->sharedNumber; i++) {
                                int wert = (buffer[aktBuffer]->BQ->usedSpace+buffer[aktBuffer]->BQ->registerSize()) 
						- buffer[aktBuffer]->sharedQueueMin;
                                if (wert < 0) {
                                        wert = 0;
                                }
                                occupiedDistributableSpace = occupiedDistributableSpace + wert;
                                aktBuffer = buffer[aktBuffer]->nextShared;
                        }
                        int minReserve = buffer[nextBuffer]->sharedQueueMin * buffer[nextBuffer]->sharedNumber;
                        if (occupiedDistributableSpace + minReserve + size <= buffer[nextBuffer]->sharedQueueTotal) {
                                return 1;
                        } else {
                                // wenn irgendwann mal Pakete betrachtet werden, die grer als 1 sind, mu hier nochmal was gemacht werden
                                return 0;
                        }
                }
        }
}
*/

void BufferQueue::push(Flit* fptr)
{
	// Es sollte Platz vorhanden sein!
	assert(freeSpace());

	internalQueue.push_back(fptr);

	rc->announceFlit(fptr);

	pushed.sign();
}

Flit* BufferQueue::front() const
{
	if(!internalQueue.empty())
		return internalQueue.front();

	return NULL;
}

Flit* BufferQueue::back() const
{
	if(!internalQueue.empty())
		return internalQueue.back();

	return NULL;
}


Flit* BufferQueue::raw_pop()
{
	Flit* tmp = NULL;

	if(!internalQueue.empty())
	{		
		// Annahme: Zaeler sollte anzeigen, dass Flit vorhanden
		assert(spaceInUse()>0);
		
		tmp = internalQueue.front();
		internalQueue.pop_front();

		popped.sign();
	}

	return tmp;
}

Flit* BufferQueue::pop()
{
	Flit* tmp = NULL;
	
	if(!internalQueue.empty())
	{		
		// Annahme: Zaeler sollte anzeigen, dass Flit vorhanden
		assert(spaceInUse()>0);
		
		tmp = internalQueue.front();
		
		internalQueue.pop_front();
		rc->eraseFlit(tmp);

		popped.sign();
	}
	return tmp;
}


void BufferQueue::clear()
{
	// Annahme: Flitregister existiert
	assert(rc);

	rc->reset();
	internalQueue.clear();
}


bool BufferQueue::empty() const
{
	return internalQueue.empty();
}


unsigned int BufferQueue::freeSpace() const
{
	// Annahme: Belegter Plaetze uebersteigen nicht Kapazitaet
	// assert(queueSpace() >= spaceInUse());
	
	// In gewissen Fllen knnen Puffer nach der Deadlock-Auflsung
	// berfllt sein. Deshalb ist hier der Assert auskommentiert.
	if(queueSpace() < spaceInUse()) return 0;

	return queueSpace()-spaceInUse();
}


unsigned int BufferQueue::estimatedSpace() const
{
	Flit* first = front();
	
	if(!first || first->head())
		return freeSpace();

	return freeSpace() + (first->Owner()->size() - first->Offset()); 
}


unsigned int BufferQueue::spaceInUse() const
{
	return internalQueue.size();
}


// einzige Stelle, wo maxSpace abgerufen werden kann.
unsigned int BufferQueue::queueSpace() const
{
	if(!infinite)
		return maxSpace;
	
	return INT_MAX; // sehr grosser Puffer. UINT_MAX = 2*INT_MAX
}


bool BufferQueue::resize(unsigned int size)
{
	if(size<1 || !empty())
		return false;
		
	maxSpace = size;
	return true;
}

