#include <configuration.h>
#include <packetforwarding.h>
#include <sourcebuffer.h>
#include <analyser.h>
#include <macros.h>

#include <environment/analysis.h>

#include <cassert>


Configuration::Configuration(const DOMElement *scope) : XMLElement(scope)
{
	_pf = NULL;
	_settings = NULL;
	
	_index = 0;
	
	_nextConfig = NULL;
	_nextBuffer.clear();
	_nextTarget.clear();
	
	_operationPhase = UINT_MAX;
	_drainPhase = 0;
	_reconfigPhase = 0;

	_dynamic = false;
}


Configuration::~Configuration()
{
	if(_pf)
		delete _pf;
	
	// Komponenten freigeben
	for(RTTIMap::iterator it=_components.begin(); it!=_components.end(); it++)
	{
		for(unsigned int i=0; i<it->second.size(); i++)
		{
			delete it->second[i];
		}
	}
		
	// Analyser freigeben
	for(unsigned int i=0; i<_analysers.size(); i++)
		delete _analysers[i];

	_analysers.clear();
	_components.clear();

	_nextBuffer.clear();
	_nextTarget.clear();
}


bool Configuration::init(const DOMElement *scope)
{
	if(scope)
		setScope(scope);

	assert(Scope());

	//
	// Mehrfache Initialisierung fuer Simulationssequenzen moeglich
	//
	
	// Komponenten initialsieren
	for(RTTIMap::iterator it=_components.begin(); it!=_components.end(); it++)
	{
		for(unsigned int i=0; i<it->second.size(); i++)
		{
			//std::cerr << "Init: " << it->second[i]->toString() << std::endl;
			
			it->second[i]->init();
		}
	}
		
	//
	// Ab hier nur einmalige Initialisierung
	//

	if(!_pf)
	{	
		// Analyser initialisieren
		for(unsigned int i=0; i<_analysers.size(); i++)
			_analysers[i]->init();
		
		assert(_settings);

		// Bitmasken initialisieren
		ComponentVector &sources = _components[Component::SOURCE];
		for(unsigned int i=0; i<sources.size(); i++)
			((SourceBuffer*)sources[i])->checkReachableTargets();

		// Koroutine initialisieren
		_pf = (PacketForwarding*)FactoryElement::Factory::self().
					create(getAttribute("forwardingType", _settings));

		assert(_pf);

		_pf->init(_settings);
	}
		
	return true;
}


void Configuration::advance()
{
	_pf->start();

	Analysis::StartGathering();
	
	for(unsigned int i=0; i<_analysers.size(); i++)
	{
		_analysers[i]->observe();
	}

	Analysis::StopGathering();
}


void Configuration::reset()
{
	// Komponenten zuruecksetzen 
	for(RTTIMap::iterator it=_components.begin(); it!=_components.end(); it++)
	{
		for(unsigned int i=0; i<it->second.size(); i++)
		{
			//std::cerr << "Reset: " << it->second[i]->toString() << std::endl;
			
			it->second[i]->reset();
		}
	}

#ifdef NDEBUG			
	const ComponentVector& b = Components(Component::BUFFER);
	const ComponentVector& t = Components(Component::TARGET);
	const ComponentVector& s = Components(Component::SOURCE);

	for(unsigned int i=0; i<b.size(); i++)
	{
		assert(((Buffer*)b[i])->BQ->empty());

		assert(!((Buffer*)b[i])->BQ->Register()->firstFlit());
		assert(!((Buffer*)b[i])->BQ->Register()->getPacket());
	}
	
	for(unsigned int i=0; i<s.size(); i++)
	{
		assert(((Buffer*)s[i])->BQ->empty());
		
		assert(!((Buffer*)s[i])->BQ->Register()->firstFlit());
		assert(!((Buffer*)s[i])->BQ->Register()->getPacket());
	}
	
	for(unsigned int i=0; i<t.size(); i++)
	{
		assert(((Buffer*)t[i])->BQ->empty());
	}
#endif
}


void Configuration::transferPackets()
{
	_ERROR("Not implemented yet!");
}

