#include <ffcontroller.h>
#include <packetforwarding.h>
#include <wirematrixiterator.h>
#include <configuration.h>
#include <arbiter.h>
#include <wire.h>
#include <macros.h>

#include <cassert>

REGISTER_ELEMENT(FFController, "FirstFit")

bool FFController::activate(Wire* outwire) 
{
	// Index des Arbiters ist Ausgangs-Index der Leitung
	unsigned int index = outwire->getOutIndex();
	// relevante Eingaenge fuer ausgehende Leitung
	std::vector<Wire*> competitors;

	// Annahme: Ein Arbiter existiert.
	assert(arbiter.size() > index);
	// Annahme: Zugriff auf Schaltmatrix moeglich.		
	assert(_wm != NULL);
	
	// Arbiter zu Leitung aus Liste
	Arbiter *ar = arbiter[index];

	// Annahme: Noch kein Scheduling im aktuellen Takt.
	assert(!ar->valid());
	
	// Matrix nach relevanten Eingaengen durchsuchen	
	WireMatrixIterator* it = _wm->iterateInWires(outwire);
	while (it->hasNext()) 
	{
		it->next();
		
		// Test, ob Paket vermittelt werden kann
		if(it->set(Coroutine()->forwarding())
				== WireMatrixModifier::SKIPPED)
		{
			// Der Test der Backpressure-Bedingung konnte nicht
			// durchgefuehrt werden, weil unzureichende Informationen
			// zum aktuellen Ausgang vorliegen!
			_ERROR("Not implemented yet", 111);
		}
		
		WireMatrixEntry current = it->value();
		if (competitor(current))
			competitors.push_back(current.getInWire());
	}
	
	// letztes Scheduling verwerfen
	ar->Forget();
        
	// neues Scheduling, falls relevante Eingaenge vorhanden
        if (competitors.size()) 
	{
		// Scheduling
		ar->Schedule(competitors);
	
		// Entscheidung akzeptieren
		ar->Accept(competitors);
	}

	// Arbiter fuer Takt signieren
	ar->sign();
	return true;
}

