#ifndef _FFCONTROLLER_H__
#define _FFCONTROLLER_H__

#include <arbitercontroller.h>

//! Redundanzverhalten First-Fit.
/*!
 * Diese Klasse implementiert die abstrakte Basisklasse ArbiterController
 * und realisiert damit das Redundanzverhalten First-Fit. Das bedeutet, dass
 * die Entscheidung eines Arbiters fuer einen Ausgang akzeptiert wird
 * und Verlierer sich noch um moegliche weitere Ausgaenge bewerben koennen.
 * Deckt der Ausgang alle verbleibenden Ziele des ausgewaehlten Einganges
 * ab, so nimmt dieser Eingang an keinem Scheduling mehr teil. Fuer einen
 * siegreichen Eingang wird gewaehrleistet, dass ein entsprechendes Paket,
 * auch den Ausgang im aktuellen Takt benutzen kann.
 * 
 * @see ArbiterController
 *
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class FFController : public ArbiterController
{
public:

	//! Standard-Konstruktor
	FFController(const DOMElement* dom=NULL) : ArbiterController() {}

	//! Implementierung der abstrakten Basisklasse ArbiterController.
	/*! 
	 * Diese Methode implementiert das Redundanzverhalten First-Fit. Fuer
	 * den uebergebenen Ausgang werden alle relevanten Eingaenge ermittelt
	 * und ein Scheduling durch Arbiter des Ausganges durchgefuehrt. Der
	 * dadurch ermittelte Gewinner kann auf den Ausgang geschaltet werden, 
	 * unabhaengig davon, ob dieser Ausgang optimal ist.
	 *
	 * @param output Der zu vermittelnde Ausgang
	 * @return true Bei Erfolg.
	 */
	virtual bool activate(Wire* output);	
};

#endif
