#include <distribution.h>
#include <environment/prng.h>

#include <cassert>
#include <limits>

Distribution::Distribution() : XMLElement()
{
	initialized = false;
	attrprefix = NULL;
	distelements = 0;
	varray_sum=0;

	carray = NULL;
	//backup = NULL;
}

void Distribution::setAttributePrefix(const char* prefix)
{
	if(prefix)
	{
		// aufraeumen
		if(attrprefix)
			delete [] attrprefix;
			
		// Praefix setzen
		attrprefix = new char[strlen(prefix)+1];
		strcpy(attrprefix, prefix);	
	}
}

int Distribution::choose()
{
	if (!initialized)
		_ERROR("Distribution not initialized!");

	assert(carray);

	try {
		std::numeric_limits<double> double_info;
		last = carray->choose_higher(UniformDouble (0, carray->getNorm()-double_info.min()));
		return id.at(last);

	} 
	catch (CumulativeException ce) {
		_ERROR("Unrecovereable error occured");
		_ERROR(ce.what());
	}

	return -1;
}


void Distribution::remove_last() 
{
	carray->remove(last);
	id.erase(id.begin()+(last));
}

void Distribution::initArray(std::vector<double> &values)
{
	if(carray)
	{
		delete carray;
		carray = NULL;	
	}
	
	carray = new CumulativeArray<double>(values);
	carray->normalize();
	backup = (*carray);
	
	varray.clear();
	varray_sum = 0.0;
	for(unsigned int i=0; i<values.size(); i++)
	{
		varray.push_back(values[i]);
		varray_sum += varray[i];
	}	
	
	initialized = true;
}

void Distribution::reset() 
{
	if(carray)
	{
		(*carray) = backup;
	}

	id.clear();
	for (unsigned int i=0; i<distelements; i++)
		id.push_back(i);
}

double Distribution::probability(unsigned int distelem) const
{
	//assert(distelements == varray.size());
	
	if(distelem >= varray.size())
		return -1;

	if(!varray_sum) return 0;

	return varray[distelem]/varray_sum;
}

Distribution::~Distribution() {}
