#include <formula.h>
#include <utility/xmlhelper.h>

#include <config.h>
#include <cassert>

REGISTER_ELEMENT(FormulaDistribution, "Formula")


bool FormulaDistribution::init(const DOMElement* scope)
{
	std::stringstream buffer;
	std::string attribute;
	std::vector<DOMElement*> fvars;
	DOMElement* top;
	
	setScope(scope);

	// XML-Element muss gesetzt sein 
	assert(Scope());
	// Prefix muss gesetzt sein
	assert(attributePrefix());
	
	// Attribut mit Verweis auf Formel auslesen
	buffer << attributePrefix() << "_formula";
	buffer >> attribute;
	buffer.clear();
	
	const char* tmp = conv(Scope()->getAttribute(conv(attribute.c_str())));
	
	// Ebene mit Formel-Definitionen
	top = Scope()->getOwnerDocument()->getDocumentElement();
	top = getFirstChildByName(top, "net", equals);

	assert(top);

	// passende Formel-Variable suchen
	fvars = getChildElementsByName(top, "formulavar", false, equals);
	for(unsigned int i=0; i<fvars.size(); i++)
	{
		if(!strcmp(tmp, conv(fvars[i]->getAttribute(conv("formulaVar")))))
		{
			tmp = conv(fvars[i]->getAttribute(conv("definition")));	
			stringToParse = new char[strlen(tmp)+1];
				strcpy(stringToParse, tmp);
		}	
	}
	
	if(!stringToParse)
		_ERROR("Invalid formula variable - " << tmp << "!");
	
	// linke Grenze
	buffer << attributePrefix() << "_leftBorder";
	buffer >> attribute;
	buffer.clear();

	left = parseAttribute(Scope(), attribute.c_str());

	// rechte Grenze
	buffer << attributePrefix() << "_rightBorder";
	buffer >> attribute;
	buffer.clear();

	right = parseAttribute(Scope(), attribute.c_str());
	
	// Verteilung anhand von Formel und Grenzen initialisieren
	parseString();
	
	return true;
}

int FormulaDistribution::parseString()
{
	if(Initialized()) return 0;
	
	char* delims = ";";
	char* expr = strtok(stringToParse,delims);
	std::vector<double> values;
	
	while(expr) 
	{	
		parse(expr); 
		if (PError())
                	_ERROR("Parse error while parsing expression ' " << expr << " ' : " << PError());

		expr = strtok(NULL, delims);
	}

	// create a cumulative array
	if (distelements == 0)
			_ERROR("Cannot distribute zero elements!");
	if ((!left && !right) || (left >= right))
			_ERROR("Left and right borders of a distribution has to be greater than 0");

	double interval = fabs( ( right - left)/ (double)(distelements-1) );
	int err = 0;
	for (unsigned int elem=0; elem<distelements; elem++) 
		values.push_back(mp_eval_exp(left + elem*interval, &err));

	initArray(values);

	assert(!PError());
	
	return 0;
}


