#ifndef __FORMULA_DISTRIBUTION__
#define __FORMULA_DISTRIBUTION__

#include <distribution.h>

//! Verteilung mit Definition ueber Formel.
/**
 * Diese Klasse definiert eine Verteilung, die ber eine mathematische Formel
 * spezifiziert wird. Die zu verteilenden Elemente werden gleichfrmig unter
 * der Kurve der Funktion innerhalb der Grenzen (einschlielich) verteilt und
 * daraus eine Wahrscheinlichkeitsverteilung bestimmt.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class FormulaDistribution : public Distribution
{
	public:
 		//! Standard-Konstruktor.
		FormulaDistribution() : Distribution(), stringToParse(NULL) {}
			
		//! Implementierung der abstrakten Basisklasse XMLElement.
		/**		 
		 * Der relevante Teil des XML-Elements wird untersucht und die
		 * zu verwendenden Wahrscheinlichkeiten anhand einer Formel und 
		 * zugehrigen Grenzen initialisiert. 
		 *
		 * @param scope XML-Element zu diesem Factory-Element.
		 * @return true
		 */
          	bool init(const DOMElement* scope=NULL);
	
	protected:

		//! Verteilung aus Formeldefinition erstellen.
		/**
		 * Diese Methode wird verwendet, um eine Formeldefinition zu
		 * parsen und anhand der gegebenen Grenzen einer Verteilung
		 * zu bestimmen.
		 *
		 * @return 0, falls erfolgreich.
		 */
		int parseString();
		
	private:

		//! String mit Formeldefinition.
		char* stringToParse;
	
		//! Linke Grenze der Funktion.
        	double left;
	
        	//! Rechte Grenze der Funktion.
       		double right;
};

#endif
