#include <environment.h>
#include <macros.h>

#include <utility/valuestring.h>
#include <environment/pluginmanager.h>
#include <environment/parser/cmdparser.h>
#include <environment/parser/configparser.h>

using namespace std;

Environment* Environment::_instance = NULL;


void Environment::Setup(const char* id, int argc, char** argv)
{
	if(_instance)
	{
		_instance->finalize();
		_ERROR("Environment already set up! ");
	}
	
	_instance = (Environment*)FactoryElement::Factory::self().create(id);

	if(!_instance)
		_ERROR("Failed to set up environment (" << id << ")!");

	if(!_instance->initialize(argc,argv))
	{
		_instance->finalize();
		
		exit(0);
	}
		
	// Vollstaendige Konfiguration ausgeben
	assert(_instance->isSet(_SHOW_VERBOSE_));
	if((int)_instance->getValue(_SHOW_VERBOSE_))
	{
		cout << "---" << std::endl;
		_instance->print();
		cout << "---" << std::endl;
	}
}


bool Environment::initialize(int argc, char** argv)
{
	assert(_instance);
	
	stringstream path;
	CmdParser *cmdparser = NULL;
	ConfigParser *cfgparser = NULL;

	// Kommandozeilenparser
	cmdparser = _instance->createCmdParser();
	// Default Konfigurationsdatei
	path << PACKAGE_DATA_DIR << "/" << "config.xml";
	cmdparser->addValue(_CONF_FILE_, ValueString(path.str()));
	
	// Kommandozeile parsen
	if(!cmdparser->parseArgs(argc, argv)) return false;
	
	// Hinweise zu Benutzung ausgeben und beenden
	if (cmdparser->isSet("SHOW_USAGE"))
	{
		cmdparser->printUsage(argc, argv);
		return false;
	}
		
	// Plugins suchen und ausgeben
	PluginManager* plugins = new PluginManager;
	std::vector<string> pluginList = plugins->scanForPlugins();
	if (cmdparser->isSet("SHOW_PLUGINS"))
	{
		std::cout << "Printing plugin list - ignoring other options ...\n" << std::endl;
		for (std::vector<string>::iterator it = pluginList.begin(); it<pluginList.end(); it++)
			cout << (*it) << std::endl;
		return false;
	}
	
	// Keine Simulationsbeschreibung - Hinweise zu Benutzung ausgeben und beenden
	if(!cmdparser->isSet(_XML_INPUT_))
	{
		cmdparser->printUsage(argc, argv);
		return false;
	}

	// Konfiguration aktualisieren
	cmdparser->updateConfig(_instance);

	// Konfigurationsdatei parsen	
	cfgparser = new ConfigParser();
	if(!cfgparser->parseFile(((string)_instance->getValue(_CONF_FILE_)).c_str())) return false;

	// Konfiguration vervollstaendigen
	cfgparser->completeConfig(_instance);
	
	// Nice-Level setzen
	assert(_instance->isSet(_NICE_LEVEL_));
	if (nice (_instance->getValue(_NICE_LEVEL_)) == -1)
		_WARNING("Failed to set nice level to " << (string)_instance->getValue(_NICE_LEVEL_) << "!")
				
	delete cmdparser;
	delete cfgparser;

	return true;
}


void Environment::Halt()
{
	if(!_instance) _ERROR("No environment set up!");
		
	_instance->finalize();

	delete _instance;

	_instance = NULL;
}


void Environment::setParameter(string name, double value)
{
	if(!_instance) _ERROR("No environment set up!");

	stringstream vstream;
	vstream << value;
	
	_instance->addValue(name, ValueString(vstream.str()));
}
