#ifndef __CINSIMANALYSIS_H__
#define __CINSIMANALYSIS_H__

#include <environment/analysis.h>

class Variable;

//! Statistische Auswertung fuer Messwerte.
/*!
 * Diese Klasse verwaltet die Messwerte einer Simulation in Form
 * von Messvariablen. Jede Variable fuehrt eine Auswertung bezueglich
 * Mittelwert oder p-Quantil.
 *
 * @see Variable
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class CINSimAnalysis : public Analysis
{
	
public:

	//! Konstruktor.
	CINSimAnalysis() : _variables(NULL), _count(0) {};
	
	//! Destruktor.
	~CINSimAnalysis();
	
	//! Implementierung der Basisklasse Analysis.
	/*!
	 * Es werden Messvariablen fuer die ueber \a list spezifizierten Messungen
	 * angelegt und die Auswertung damit initialsiert. Ein erneuten Aufruf
	 * gibt alle Variablen frei und erzeugt neue.
	 * 
	 * @see Analysis
	 */
	bool declareVariables(const VariableInfo* list, unsigned int count);

	//! Implementierung der Basisklasse Analysis.
	/*!
	 * Der Aufruf dieser Methode aktualisiert die Messvariable \a _variables[var-1] 
	 * mit dem Messwert \a value. Wurde ein Checkpoint erreicht so wird das Ergebnis
	 * \a result aktualisiert.
	 * 
	 * @see Analysis
	 */
	bool advanceVariable(unsigned int var, double value, Result& result);
	
	//! Implementierung der Basisklasse Analysis.
	/*!
	 * 
	 * @see Analysis
	 */
	void finishVariable(unsigned int var);
	
	
private:

	//! Variablen fuer Messwerte.
	Variable** _variables;
	//! Anzahl der Variablen.
	unsigned int _count;
};

#endif
