#include <independentmean.h>
#include <mathematics.h>

#include <cmath>
#include <iostream>
#include <cassert>

#define _CHKPNT_ 512

IndependentMean::IndependentMean(double confidence) : Estimation(confidence)
{
	_ssize=0.0;
	_ssquares=0.0;
	_ssum=0.0;

	_zdist = Z((1.0 - _confidence)/2.0);
}

inline bool IndependentMean::checkPoint() const
{
	return ((unsigned int)_ssize) % _CHKPNT_ == 0;
}

bool IndependentMean::advance(double value)
{
	_ssquares += (value * value);
	_ssum += value;
	_ssize += 1;

	if(checkPoint())
	{
		_estimate = _ssum / _ssize;
		_variance = (_ssquares - 2.0*_estimate*_ssum + _ssize*_estimate*_estimate) / 
			(_ssize*(_ssize-1));
		
		_error = (_zdist * sqrt(_variance)) / (_estimate > 0.0 ? _estimate : DBL_MIN);
		_freedom = _ssize;

		return true;
	}

	return false;
}

void IndependentMean::reset()
{
	Estimation::reset();

	_ssize = 0.0;
	_ssquares = 0.0;
	_ssum = 0.0;
}
