#include <environment/prng.h>
#include <macros.h>

using namespace std;

string PRNG::_factoryid = "CMRG";
unsigned int PRNG::_streamid = 0;
PRNG* PRNG::_instance = NULL;


bool PRNG::createInstance()
{
	if(_instance)
		delete _instance;
	
	_instance = (PRNG*)FactoryElement::Factory::self().create(_factoryid);
	
	if(!_instance)
		_ERROR("Failed to create instance for id " << _factoryid << "!");

	_instance->uniqueStreamId(_streamid);

	return true;
}

void PRNG::setFactoryId(const string& id)
{
	if(!Instance())
	{
		_factoryid = id;
	}
}

void PRNG::setStreamId(unsigned int id)
{
	if(Instance())
	{
		_instance->uniqueStreamId(id);
	}
	_streamid = id;
}

double PRNG::normal(double mean, double variance)
{
	double v1, v2, w, z1; 
	static double z2 = 0.0;
	if (z2 != 0.0) 
	{
		z1 = z2; 
		z2 = 0.0;
	}
	else 
	{
		do 
		{
			v1 = 2.0 * uniformDouble_01() -1.0; 
			v2 = 2.0 * uniformDouble_01() - 1.0; 
			w = v1 * v1 + v2 * v2;
		}
		while (w >= 1.0);

		w = sqrt((-2.0*log(w))/w); 
		z1 = v1 * w; 
		z2 = v2 * w;
	}
	return mean + z1 * sqrt(variance);
}
