#ifndef __CMRG_H__
#define __CMRG_H__

#include <environment/prng.h>
#include <rngstream/RngStream.h>

//! Generator fuer Pseudo-Zufallszahlen basierend auf RngStreams.
/*!
 * Diese Klasse implementiert die abstrakte Basisklasse PRNG mit Methoden
 * der Bibliothek RngStreams. Diese verwendet einen Combined Multiple
 * Recursive Generator (CMRG) und erlaubt die Spezifikation von sehr 
 * langen und unabhaengigen Folgen von Zufallszahlen.
 *
 * @see PRNG
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class CMRG : public PRNG
{
	public:

		//! Konstruktor.
		/**
		 * Der Aufruf des Konstruktors erzeugt einen Combined Multiple
		 * Recursive Generator (mrg32k3a).
		 */
		CMRG(); 

		//! Destruktor.
		/**
		 * Die interne Generator-Struktur wird freigegeben.
		 */
		~CMRG();

		//! Implementierung der Basisklasse PRNG.
		/*!
		 * @see PRNG::uniformInt()
		 */
		unsigned int uniformInt(unsigned int start, unsigned int end);
		
	protected:
		
		//! Implementierung der Basisklasse PRNG.
		/*!
		 * @see PRNG::uniqueStreamId()
		 */
		void uniqueStreamId(unsigned int uid);
			
	private:
		
		//! Interner Zustand.
		RngStream stream;	
};

#endif
