#ifndef __MTWISTER_H__
#define __MTWISTER_H__

#include <environment/prng.h>
#include <dc.h>

//! Dynamisch erzeugte Generatoren fuer Pseudo-Zufallszahlen.
/*!
 * Diese Klasse implementiert die abstrakte Basisklasse PRNG durch Verwendung
 * dynamisch erzeugter Mersenne Twister Generatoren. Zu jeder eindeutigen ID
 * wird ein unabhaengiger Generator mit einer sehr langen Folge von Zufallszahlen
 * bereitgestellt.
 *
 * @see PRNG
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class MTwister : public PRNG
{
	public:

		//! Konstruktor.
		/*!
		 * Der Aufruf des Konstruktors erzeugt einen Mersenne Twister.
		 */
		MTwister(); 

		//! Destruktor.
		/**
		 * Die interne Generator-Struktur wird freigegeben.
		 */
		~MTwister();
		
		//! Implementierung der Basisklasse PRNG.
		/*!
		 * @see PRNG::uniformInt()
		 */
		unsigned int uniformInt(unsigned int, unsigned int);
	
	protected:
		
		//! Implementierung der Basisklasse PRNG.
		/*!
		 * @see PRNG::uniqueStreamId()
		 */
		void uniqueStreamId(unsigned int uid);
			
	private:
		
		//! Interner Generator.
		mt_struct *mtwister;	
};

#endif
