#include <test_prng.h>

#include <iostream>
#include <iomanip>
#include <fstream>
#include <stdlib.h>

CPPUNIT_TEST_SUITE_REGISTRATION( Test_PRNG );

#define DUMMY_ID "FooBar"

using namespace std;

void Test_PRNG::tearDown()
{
	if(prng)
		delete prng;
}

void Test_PRNG::init_check()
{	
	srand( time(0) );
	unsigned int id = rand();
	std::string defaultid;
	
	CPPUNIT_ASSERT( !PRNG::Instance() );
	CPPUNIT_ASSERT( PRNG::StreamId() == 0);

	defaultid = PRNG::FactoryId();
	
	PRNG::setFactoryId(DUMMY_ID);
	
	CPPUNIT_ASSERT( !PRNG::Instance() );
	CPPUNIT_ASSERT( PRNG::FactoryId() == DUMMY_ID);

	PRNG::setFactoryId(defaultid);
	PRNG::createInstance();
	
	CPPUNIT_ASSERT( PRNG::FactoryId() == defaultid);
	CPPUNIT_ASSERT( PRNG::Instance() );
	
	PRNG::setStreamId(id);
			
	CPPUNIT_ASSERT( PRNG::StreamId() == id );	
}

void Test_PRNG::reproduce_check()
{	
	srand( time(0) );
	unsigned int id = rand();
	std::vector<unsigned int> ruints;

	CPPUNIT_ASSERT( prng );

	prng->uniqueStreamId(id);
	
	// Zufallszahlen aus Stream id
	for(unsigned int k=0; k<100; k++)
	{
		ruints.push_back(prng->uniformInt(0,UINT_MAX));
	}
	
	prng->uniqueStreamId(id);
	
	// Gleiche Zufallszahlen aus Stream id
	for(unsigned int k=0; k<100; k++)
	{
		CPPUNIT_ASSERT( prng->uniformInt(0,UINT_MAX) == ruints[k] );
	}
}

void Test_PRNG::badargs_check()
{
	CPPUNIT_ASSERT( prng );
	
	CPPUNIT_ASSERT( prng->uniformInt(0,0) == 0 );
	CPPUNIT_ASSERT( prng->uniformDouble(0.0,0.0) == 0.0 );
}

void Test_PRNG::generate_check()
{
	// Zufaellige Id
	srand(time(0));
	unsigned int id = rand();
	
	CPPUNIT_ASSERT( prng );
	
	prng->uniqueStreamId( id );
	
	std::stringstream filename; filename << dataPrefix() << id << ".dat";
	std::ofstream ofl(filename.str().c_str());
		
	CPPUNIT_ASSERT( ofl );
		
	double rdouble1;		// random double from ]0,1[
	double rdouble2;		// random double from [100.0,110.0]
	unsigned int ruint1;	// random int from [0,UINT_MAX]
	unsigned int ruint2;	// random int from [1,10]
	
	ofl << "## Generator " << id << " (" << factoryId() << ")" << endl; 
	
	// 10000 Zufallszahlen generieren
	for( unsigned int k = 0; k < 10000; k++ )
	{	
		rdouble1 = prng->uniformDouble();
		rdouble2 = prng->uniformDouble(100.0,110.0);
		ruint1 = prng->uniformInt(0,UINT_MAX);
		ruint2 = prng->uniformInt(1,10);
		
		CPPUNIT_ASSERT( rdouble1 > 0.0 && rdouble1 < 1.0 );
		CPPUNIT_ASSERT( rdouble2 >= 100.0 && rdouble2 <= 110.0 );
		CPPUNIT_ASSERT( ruint2 > 0 && ruint2 < 11 );

		ofl << left << setw(20) << rdouble1;
		ofl << left << setw(20) << rdouble2;
		ofl << left << setw(20) << ruint1;
		ofl << left << setw(20) << ruint2;
		ofl << endl;
	}
	ofl.close();
}

