#include "ParserImpl.h"


struct init arith_fncts[]
=
{
    { "sin",    sin    },
    { "asin",   asin   },
    { "cos",    cos    },
    { "acos",   acos   },
    { "tan",    tan    },
    { "atan",   atan   },
    { "ln",     log       },
    { "log",    log10     },
    { "exp",    exp       },
    { "sqrt",   sqrt      },
    { "abs",    fabs      },
    { "tanh",   tanh   },
    { "atanh",  atanh  },
    { "sinh",   sinh   },
    { "asinh",  asinh  },
    { "cosh",   cosh   },
    { "acosh",  acosh  },
    { "cot",    mp_cot    },
    { "csc",    mp_csc    },
    { "sec",    mp_sec    },
    { "sgn",    mp_sign   },
    { "sign",   mp_sign   },
    { "lgamma", lgamma    },
    { "erf",    erf       },
    { "erfc",   erfc      },
    { 0,        0         }
};

struct initc constants[]
  = {
      { "e",  M_E   },
      { "pi", M_PI  },
      { "a",  0     },
      { 0,    0     }
  };

ParserImpl::ParserImpl() {
	sym_table = NULL;
	//TODO: cannot parse a files contents yet
	stringToParse = new char[512];
	//sprintf(stringToParse, "sin(cos(sin(2*x/8 - 1))*1000/10)");
	
	// build a clean emty symbol stack
	init_stack();
	// put known functions and constants into an extra symboltable
	initSymbolTable();
	

}

symrec* ParserImpl::putsym (char* sym_name, int sym_type)
{
    symrec *ptr = new symrec;
    ptr->name = new char[strlen (sym_name) + 1];
    strcpy (ptr->name,sym_name);
    ptr->type = sym_type;
    ptr->value.var = 0; // set value to 0 even if fctn. 
    ptr->next =sym_table;
    sym_table = ptr;
    return ptr;
}

symrec* ParserImpl::getsym (char* sym_name)
{
    symrec *ptr = NULL;
    for (ptr = sym_table; ptr != NULL;
         ptr = ptr->next)
        if (strcmp (ptr->name,sym_name) == 0)
            return ptr;
    return 0;
}


void ParserImpl::initSymbolTable ()  /* puts arithmetic functions in table. */
{
	
    int i;
    symrec *ptr;

    for (i = 0; arith_fncts[i].fname != 0; i++)
    {
        ptr = putsym (arith_fncts[i].fname, FNCT);
        ptr->value.fnctptr = arith_fncts[i].fnct;
    }

    for (i = 0; constants[i].cname != 0; i++)
    {
        ptr = putsym (constants[i].cname, CONST);
        ptr->value.var = constants[i].cvalue;
    }
}

void ParserImpl::yyerror(char * msg)
{
/*	cerr << yylloc.first_line << ":" << yylloc.first_column <<  ": "
		<< msg << " : <" << yylloc.text << ">" <<  endl;
*/

cerr << msg <<  endl;
		
}

/* Komplexer Lexer
 * Die Unterscheidung von Funktionen und Variablen ist leider
 * ueber eine grammatik zu kompliziert und kann nativ deutlich 
 * einfacher ausgedrueckt werden.
 */
int ParserImpl::yylex ()
{
        if (flagend == 1) { flagend = 0; return (0); }

    
    if (!initial){ ++stringToParse; }
    
    initial = 0;
 
    
    if (*stringToParse == ' ' || *stringToParse == '\t')
    {
	/* Ignore whitespace, get first nonwhite character.  */
	while (*++stringToParse == ' ' || *stringToParse == '\t')
	    ;
    }
    
    if (*stringToParse == '\0')
    {
	flagend = 1;
	return ('\n');
    }
    
	/* Char starts a number => parse the number.         */
    if (*stringToParse == '.' || isdigit (*stringToParse) /*|| *c == ','*/)
    {
	yylval.val = strtod(stringToParse, (char **)NULL);
	while (*stringToParse == '.' || isdigit (*stringToParse)) ++stringToParse;
	--stringToParse;
	return NUM;
    }

    /* Char starts an identifier => read the name.       */
    if (isalpha (*stringToParse))
    {
	symrec *s;
	char *symbuf = 0;
	int length = 0;
	int i;
	
	/* Initially make the buffer long enough
	   for a 40-character symbol name.  */
	if (length == 0)
	    length = 40, symbuf = (char *)malloc (length + 1);
	
	i = 0;
	do
        {
	    /* If buffer is full, make it bigger.        */
	    if (i == length)
            {
		length *= 2;
		symbuf = (char *)realloc (symbuf, length + 1);
            }
          /* Add this character to the buffer.         */
	    symbuf[i++] = tolower(*stringToParse);
	    /* Get another character.                    */
	    ++stringToParse;
        }
	while (*stringToParse != '\0' && isalnum (*stringToParse));
	
	--stringToParse;
	
	symbuf[i] = '\0';
	
	
	s = getsym (symbuf);
	
	
	if (s == 0)
	{
	    /* it isn't a function - lets hope its a variable */
	    if (strlen (symbuf) == 1)
	    { /* OK */
		s = putsym (symbuf, VAR);
	    }
	    else
	    { /* it's ok */
		/* The user entered a string which is longer than one char but not a function */
		char *namesym;
		namesym = (char *) malloc (strlen(symbuf) + 20);
		sprintf (namesym, "%s \"%s\"", "Unknown token", symbuf);
		yyerror_real (namesym);
		free (namesym);
		flagend = 1;
		return ('\n');
	    }
	}	
	yylval.tptr = s;
	return s->type;
    }
    
    
    
    /* Any other character is a token by itself.        */
    return (*stringToParse);
}    

void ParserImpl::parse() {
	yyparse();
}

void ParserImpl::parse(char* arg) {
	initial = 1;
	strcpy(stringToParse, arg);
	parse();
};

/* Cotangent */
double mp_cot (double x)
{
    return (1/tan(x));
}


/* Cosecant */
double mp_csc (double x)
{
    return (1/sin(x));
}


/* Secant */
double mp_sec (double x)
{
    return (1/cos(x));
}

/* Sign */
double mp_sign (double x)
{
    if (x < 0) return (-1);
    if (x == 0) return (0);
    return (1);
}

