%header{
    #include "scanner.h"
    #include <cstdlib>
    #include <iostream>
    #include "ParserInterface.h"
    #include <math.h>
    #include <map>

    using std::cerr;
    using std::cout;
using std::endl;

%}

%name Parser

%define LSP_NEEDED
%define ERROR_BODY = 0

%define INHERIT : public ParserInterface

/* Hier muss der lexer des scanners angegeben werden, so doch jemand einmal
* eine gute grammatk findet.
*/
//%define LEX_BODY {return d_scanner.yylex();}
%define LEX_BODY = 0

%define CONSTRUCTOR_CODE { flagend=0; parseerr = 0; evalerr = 0; }

%define MEMBERS                                         \
public:						\
virtual ~Parser()   {}                          \
	double mp_eval_exp (double x, int *err);\
	short PError() { return parseerr; };	\
	void mp_evalStack_clean (void);		\
private:                                            \
std::map<char, double> varTable;	\
short parseerr;					\
char errtext[80];				\
void mp_push_num(double);			\
void mp_push_fnct(symrec*);			\
void mp_push_const(symrec*);			\
void mp_push_op (short);			\
void mp_push_var (char, double);		\
struct mp_stack * mp_new_entry (const int);	\
	void mp_stack_clean (void);			\
	void mp_free_entry(const int);			\
	short evalerr;					\
    protected:						\
    struct mp_stack **entryidx[2][2]; 			\
	short initial, flagend;				\
	int yyerror_real (char*);			\
	virtual void init_stack();			\
	double mp_fact (double in);
	




%define DEBUG 1

%union {
double     val;  /* For returning numbers.                   */
symrec  *tptr;   /* For returning symbol-table pointers      */
}

%token <val>  NUM             /* Simple double precision number   */
%token <tptr> VAR FNCT CONST  /* Variable and Function            */
%type  <val>  exp

%right '='
%left OPEN    /* Multiplication        */
%nonassoc EXPNUM
%left '-' '+'
%left '*' '/'
%left NEG     /* Negation--unary minus */
%right '^'    /* Exponentiation        */



/* Grammar follows */

%%

input:   /* empty */
        | input line
;

line:
		  '\n'	{}
        | exp   '\n'  { parseerr = 0;                                     }
        | error '\n'  { if (!parseerr) { yyerror_real ("Syntax error"); } }
;

exp:      NUM                                 { mp_push_num($1); }

        | FNCT '(' exp ')' NUM %prec EXPNUM   { mp_push_fnct ($1);
	                                        mp_push_num ($5);
					        mp_push_op (MP_TOKEN_MULT);  }

		| exp ';' exp 			      { }

        | FNCT '(' exp ')' CONST %prec EXPNUM { mp_push_fnct ($1);
	                                        mp_push_const ($5);
						mp_push_op (MP_TOKEN_MULT);  }

        | FNCT '(' exp ')' VAR %prec EXPNUM   { mp_push_fnct ($1);
	                                        mp_push_var (*($5->name),
						$5->value.var);
						mp_push_op (MP_TOKEN_MULT);  }

        | CONST                               { mp_push_const ($1);          }

        | VAR                                 { mp_push_var (*($1->name), $1->value.var);              }

        | VAR '=' exp                         {  mp_push_var (*($1->name), 0); mp_push_op (MP_TOKEN_EQ); }

        | NUM VAR                             { mp_push_num ($1);
	                                        mp_push_var (*($2->name),
						$2->value.var);
						mp_push_op (MP_TOKEN_MULT);  }

        | CONST VAR                           { mp_push_const ($1);
                                                mp_push_var (*($2->name),
						$2->value.var);
						mp_push_op (MP_TOKEN_MULT);  }

        | VAR CONST                           { mp_push_const ($2);
                                                mp_push_var (*($1->name),
						$1->value.var);
						mp_push_op (MP_TOKEN_MULT);  }

        | NUM CONST                           { mp_push_const ($2);
                                               	mp_push_num ($1);
						mp_push_op (MP_TOKEN_MULT);  }

        | exp '(' exp ')' %prec OPEN          { mp_push_op (MP_TOKEN_MULT);  }

        | '(' exp ')' exp %prec OPEN          { mp_push_op (MP_TOKEN_MULT);  }

        | '(' exp ')' '-' exp %prec OPEN      { mp_push_op (MP_TOKEN_MINUS); }

        | exp '(' exp ')' exp %prec OPEN      { mp_push_op (MP_TOKEN_MULT);
	                                        mp_push_op (MP_TOKEN_MULT);  }

        | FNCT '(' exp ')'                    { mp_push_fnct ($1);           }

        | exp '+' exp                         { mp_push_op (MP_TOKEN_PLUS);  }

        | exp '-' exp                         { mp_push_op (MP_TOKEN_MINUS); }

        | exp '*' exp                         { mp_push_op (MP_TOKEN_MULT);  }

        | exp '/' exp                         { mp_push_op (MP_TOKEN_DIV);   }

        | '-' exp  %prec NEG                  { mp_push_op
						  (MP_TOKEN_UNARY_MINUS);    }

        | exp '^' exp                         { mp_push_op (MP_TOKEN_POW);   }

        | exp '!'                             { mp_push_op (MP_TOKEN_FACT);  }

	| '|' exp '|'                         { mp_push_fnct(getsym("abs")); }

        | '(' exp ')'                         { /* Intentionally empty */    }
;

/* end of grammar */

%%
int Parser::yyerror_real (char *s)  /* Called by yyparse on error */
{
    parseerr = 1;
    yyerror(s);
    sprintf (errtext, "parse error or evaluation error: %s", s);
    return (1);
}

void Parser::mp_push_num (double mpnum)
{
	
    struct mp_stack *p = mp_new_entry(0);

    p->tokentype = MP_TOKEN_CONSTANT;
    p->value = mpnum;
}


void Parser::mp_push_fnct (symrec *in)
{

    struct mp_stack *p = mp_new_entry(0);
    p->tokentype = MP_TOKEN_FNCT;
    p->symboltab = in;
}


void Parser::mp_push_const (symrec *in)
{

    struct mp_stack *p = mp_new_entry(0);
    p->tokentype = MP_TOKEN_CONST;
    p->symboltab = in;
}


struct mp_stack* Parser::mp_new_entry (const int stacknum)
{

    struct mp_stack *p = new struct mp_stack;

    if (!p)
    {
	/*ERROR*/
	return p;
    }


    if (!(*entryidx[stacknum][1]))
    {

	(*entryidx[stacknum][0]) = p;
	(*entryidx[stacknum][1]) = p;
	p->next = 0;
	p->previous = 0;
    }
    else
    {

	p->next = 0;
	p->previous = (*entryidx[stacknum][1]);

	(*entryidx[stacknum][1])->next = p;
	(*entryidx[stacknum][1]) = p;
    }


    return (p);
}

void Parser::mp_push_op (short mptoken)
{

    struct mp_stack *p = mp_new_entry(0);
    p->tokentype = mptoken;
}


void Parser::mp_push_var (char mpvar, double value)
{

    struct mp_stack *p = mp_new_entry(0);
    p->tokentype = MP_TOKEN_VARIABLE;
    p->id = mpvar;
    
    p->value = value;
}

void Parser::init_stack()
{

entryidx[0][0] = (mp_stack**) malloc(sizeof(mp_stack*));
entryidx[1][0] = (mp_stack**) malloc(sizeof(mp_stack*));
entryidx[0][1] = (mp_stack**) malloc(sizeof(mp_stack*));
entryidx[1][1] = (mp_stack**) malloc(sizeof(mp_stack*));

*entryidx[0][0] = 0;
*entryidx[1][0] = 0;
*entryidx[0][1] = 0;
*entryidx[1][1] = 0;

mp_stack_clean();


initial = 1;
flagend = 0;

}

//zerstoert den baim parsen aufgebauten baum.
void Parser::mp_stack_clean (void)
{
    struct mp_stack *p;
    if (entryidx[0][0])
    for (p=*entryidx[0][0]; p != 0; p=p->next)
    {
	mp_free_entry (0);
    }

    varTable.clear();
}

void Parser::mp_evalStack_clean (void)
{
    struct mp_stack *p;
    for (p=*entryidx[1][0]; p != 0; p=p->next)
		mp_free_entry(1);

    varTable.clear();

}

void Parser::mp_free_entry (const int stacknum)
{

    struct mp_stack *p = *entryidx[stacknum][1];
    if ((*entryidx[stacknum][0]) == p)
    {
	(*entryidx[stacknum][0]) = 0;
	(*entryidx[stacknum][1]) = 0;

	delete p;
	return;
    }
    (*entryidx[stacknum][1]) = p->previous;
    delete p;
    (*entryidx[stacknum][1])->next = 0;

}

double Parser::mp_eval_exp (double x, int *err)
{
    struct mp_stack *p, *q;
    for (p=*entryidx[0][0]; p != 0; p=p->next)
    {

	switch (p->tokentype)
	{
	case MP_TOKEN_EQ:
	    p->previous->value = (*entryidx[1][1])->previous->value;
	    varTable[p->previous->id] = (*entryidx[1][1])->previous->value;

	    (*entryidx[1][1])->previous->value = (*entryidx[1][1])->value;
	    
	    mp_free_entry (1);
	    break;
	break;
	case MP_TOKEN_VARIABLE:
	    if (p->id == 't' && !*err)
	    { 
		q = mp_new_entry(1);
		q->value = x;
	    }
	    else if (*err)
	    { 
		q = mp_new_entry(1);
		q->value=p->value;
	    } 
	    else
	    {
		q = mp_new_entry(1);
		q->value=varTable[p->id];
	    }
	    break;
	case MP_TOKEN_CONSTANT:
	    q = mp_new_entry(1);
	    q->value = p->value;
	    break;
	case MP_TOKEN_CONST: //Mathematical constant, not to be confused with numerical value above 
	    if (strcmp(p->symboltab->name, "a") == 0)
	    {
		*err = MP_ERROR_VARIABLE;
		return (0);
	    }
	    q = mp_new_entry(1);
	    q->value = p->symboltab->value.var;
	    break;
	case MP_TOKEN_PLUS:
	    (*entryidx[1][1])->previous->value = (*entryidx[1][1])->previous->value + (*entryidx[1][1])->value;
	    mp_free_entry (1);
	    break;
	case MP_TOKEN_MINUS:
	    (*entryidx[1][1])->previous->value = (*entryidx[1][1])->previous->value - (*entryidx[1][1])->value;
	    mp_free_entry (1);
	    break;
	case MP_TOKEN_MULT:
	    (*entryidx[1][1])->previous->value = (*entryidx[1][1])->previous->value * (*entryidx[1][1])->value;
	    mp_free_entry (1);
	    break;
	case MP_TOKEN_DIV:
      if (!(*entryidx[1][1])->value) {
        evalerr = 1;
        yyerror_real("Division by zero");
        return 0;
      }
	    (*entryidx[1][1])->previous->value = (*entryidx[1][1])->previous->value / (*entryidx[1][1])->value;
	    mp_free_entry (1);
	    break;
	case MP_TOKEN_FACT:
	    (*entryidx[1][1])->value = mp_fact ((*entryidx[1][1])->value);
	    break;
	case MP_TOKEN_POW:
	    (*entryidx[1][1])->previous->value = pow ((*entryidx[1][1])->previous->value, (*entryidx[1][1])->value);
	    mp_free_entry (1);
	    break;
	case MP_TOKEN_FNCT:
	    (*entryidx[1][1])->value = (*(p->symboltab->value.fnctptr))((*entryidx[1][1])->value);
	    break;
	case MP_TOKEN_UNARY_MINUS:
	    // Take the last number on the stack and reverse the sign
	    (*entryidx[1][1])->value *= (-1);
	    break;
	}
    }
    
    double val = (*entryidx[1][1])->value;
    
    mp_evalStack_clean();

	return val;
}

//TODO: das muss noch in den smbol table
double Parser::mp_fact (double in)
{
    double lgam = lgamma (in + 1);
    return signgam * exp (lgam);
}
