#ifndef _GNUPLOTOUTPUT_
#define _GNUPLOTOUTPUT_

#include <output/interface.h>
#include <output/gnuplot_i++.h>


//! Gnuplot-Ausgabe fuer Simulationssequenzen.
/*!
 * Diese Klasse implementiert das Template OutputInterface<T> fuer
 * Ergebnisse vom Typ double und erzeugt Gnuplot-Ausgaben der Mittelwerts
 * fuer jede einzelne Messung (Variable). Es muss sich dazu um transiente 
 * Simulationen oder Sequenzen (transient und stationaer) handeln.
 * Die Plots werden in Form eines Postscript-Dokumentes erzeugt.
 */
class GnuplotOutput : public OutputInterface<double>
{
public:
	
	//! Konstruktor.
	/*!
	 * Es werden das DOM-Dokument, der Name fuer das Postscript-Dokument 
	 * und der Ausgabe-Stream initialisiert.
	 * 
	 * @param doc Das DOM-Dokument zur Simulationsbeschreibung.
	 * @param file Dateiname fuer zu erzeugendes Postscript-Dokument.
	 * @param ostream Stream fuer Ausgaben.
	 */
	GnuplotOutput(const DOMDocument* doc, const std::string& file, std::ostream* ostream) 
		: OutputInterface<double>(doc, ostream), _file(file) {}

	//! Implementierung der Basisklasse OutputInterface<double>.
	void generate();

private:

	//! Dateiname fuer Postscript-Dokument.
	const std::string& _file;
};

#endif
