#include <text.h>
#include <sequencer.h>
#include <simulation.h>
#include <configuration.h>
#include <analyser.h>
#include <utility/xmlhelper.h>

#include <iomanip>
#include <iostream>
#include <cassert>

using namespace std;

#define PADDING 2
std::ostream& padding(std::ostream& os) 
{ 
	return os << std::setw(PADDING) << "" ; 
}

void TextOutput::generate()
{  
	time_t rawtime;
	unsigned int nofana = 0;
	unsigned int nofsim = 0;
	out->setf( ios_base::left );
	
	// Zugriff auf XML-Dokument notwendig
	assert(getDocument() != NULL);
	
	const AnalyserVector& analysers = Simulation::Instance()->Active()->Analysers();
	
	time(&rawtime);
	
	*out << "# " << ctime(&rawtime);
	*out << "# " << XMLString::transcode(getDocument()->getDocumentURI()) 
			<< std::endl;
	
	nofsim = Sequencer::Instance()->Run()+1;
	nofana = analysers.size();

	// steady measures
	if ( Simulation::Instance()->Type() == Simulation::STEADY_STATE ) 
	{                    
		assert(nofana && Size() == nofsim*nofana);
	
		// Informationen zu Simulation
		*out << "# Steady-state simulation" << std::endl;
		*out << std::endl;
		
		if(nofsim>1)
			*out << "# Run: 1-" << nofsim << std::endl; 
		
		// Kopfzeile fuer Ergebnisse
		*out << setw(3) << "VAR" << padding;
		*out << setw(3) << "RUN" << padding;
		*out << setw(10) << "ESTIMATE" << padding;
		*out << setw(12) << "DELTA" << padding;
		//*out << setw(12) << "VARIANCE" << padding;
		*out << setw(12) << "ERROR" << padding;
		*out << setw(10) << "VALUES" << padding;
		*out << setw(6) << "CONF" << padding;
		*out << setw(6) << "PREC" << padding;
		*out << setw(10) << "TYPE" << padding;
		*out << "DESCRIPTION";
		*out << endl;
			
		for (unsigned int m=0; m<nofana; m++) 
		{	
			for (unsigned int n=0; n<nofsim; n++)
			{
				// Parameter
				*out << setw(3) << m+1 << padding;
				// Run
				*out << setw(3) << n+1 << padding;
				// Estimate
				*out << setw(10) << getResult(n*nofana+m).Estimate() << padding;
				// Delta
				*out << setw(12) << getResult(n*nofana+m).Delta() << padding;			
				// Varianz
				//*out << setw(12) << getResult(n*nofana+m).Variance() << padding;
				// Fehler
				*out << setw(12) << getResult(n*nofana+m).Error() << padding;
				// Werte
				*out << setw(10) << getResult(n*nofana+m).Values() << padding;
				// Konfidenz
				*out << setw(6) << getResult(n*nofana+m).Conf() << padding;
				// Genauigkeit
				*out << setw(6) << getResult(n*nofana+m).Prec() << padding;
				// Typ
				*out << setw(10) << analysers[m]->Measurement() << padding;
				// Beschreibung
				*out << analysers[m]->Description();
				// Zeilenende
				*out << std::endl;
			}
		}
	}

	// transient, terminating simulation measures
	else if ( Simulation::Instance()->Type() == Simulation::TERMINATING ) 
	{
		*out << "# Terminating simulation" << std::endl;
		*out << std::endl;
		
		unsigned int tstart = Simulation::Instance()->AnalysisStart();
		unsigned int tend = Simulation::Instance()->AnalysisEnd();
		unsigned int offset = tend - tstart +1;

		assert(Size() == nofana * nofsim * offset);
                    		
		for (unsigned int n=0; n<nofsim; n++)
		{
			if(nofsim>1)
				*out << "# Run: " << n+1 << " of " << nofsim << std::endl;
			
			// Kopfzeile
			*out << setw(3) << "VAR" << padding;
			*out << setw(3) << "RUN" << padding;
			*out << setw(4) << "TICK" << padding;
			*out << setw(10) << "ESTIMATE" << padding;
			*out << setw(12) << "DELTA" << padding;			
			//*out << setw(12) << "VARIANCE" << padding;
			*out << setw(12) << "ERROR" << padding;
			*out << setw(10) << "VALUES" << padding;
			*out << setw(6) << "CONF" << padding;
			*out << setw(6) << "PREC" << padding;
			*out << setw(10) << "TYPE" << padding;			
			*out << "DESCRIPTION";
			*out << endl;

			for (unsigned int m=0; m<nofana*offset; m++) 
			{		
				// Parameter
				*out << setw(3) << m/offset+1 << padding;
				// Run
				*out << setw(3) << n+1 << padding;
				// Takt
				*out << setw(4) << m%offset+tstart << padding;
				// Estimate
				*out << setw(10) << getResult(n*nofana*offset+m).Estimate() << padding;
				// Delta
				*out << setw(12) << getResult(n*nofana*offset+m).Delta() << padding;	
				// Varianz
				//*out << setw(12) << getResult(n*nofana*offset+m).Variance() << padding;
				// Fehler
				*out << setw(12) << getResult(n*nofana*offset+m).Error() << padding;
				// Werte
				*out << setw(10) << getResult(n*nofana*offset+m).Values() << padding;			
				// Konfidenz
				*out << setw(6) << getResult(n*nofana*offset+m).Conf() << padding;
				// Genauigkeit
				*out << setw(6) << getResult(n*nofana*offset+m).Prec() << padding;
				// Typ
				*out << setw(10) << analysers[m/offset]->Measurement() << padding;				
				// Beschreibung
				*out << analysers[m/offset]->Description();
				// Zeilenende
				*out << endl;
			}
		}
	}
	else
		throw OutputException("Panic! Unknown simulation type!");

	out->unsetf( ios_base::left );
}
