#include <registercontainer.h>
#include <packet.h>
#include <bufferqueue.h>
#include <buffer.h>
#include <macros.h>

#include <assert.h>

void RegisterContainer::reset()
{	
	// Registrierungen moeglicher Empfaenger aufheben
	for(unsigned int i=0; i<customers.size(); i++)
	{	
		CustomerInfo& c_info = customers[i];
		
		//! Annahme: Wenn Paket vorhanden, dann Empfaenger registriert.
		assert(!c_info.packet || (c_info.valid && counter.registered));
		
		if(c_info.packet) c_info.packet->release();
		
		c_info.reset();
		
	}
		
	// Anliegende Flits entfernen
	while(flitlist.size())
	{
		FlitInfo& f_info=flitlist.front();
		
		// Annahme: Maske mit erreichbaren Zielen existiert.
		assert(f_info.remain);
		
		// Maske mit erreichbaren Zielen freigeben
		delete f_info.remain;
		
		// Ende eines Pakets oder letztes Flit
		if(f_info.flit->tail() || flitlist.size()==1)
		{
			// Flit zu gerade bearbeiteten Paket
			packet = f_info.flit->owner==packet ? NULL : packet;

			// Paket freigeben
			f_info.flit->owner->release();
		}
		
		flitlist.pop_front();
	}	
	
	// Anliegendes Paket freigeben
	if (packet) packet->release();
        packet = NULL;
	
	// Zaehler zuruecksetzen
	counter.reset();
}

void RegisterContainer::announceFlit(Flit* f)
{
	assert(f != NULL);

	flitlist.push_back(FlitInfo(f, f->owner->getBitmask()));

	// Paket zu Head wird in Register referenziert
	if (f->head())
	{
		f->owner->reference();
	}
	
	// Beginn eines neuen Paketes
	if (packet == NULL) 
	{
		assert(f->head());
                packet = f->owner;
		return;
        }
	
	// Flit zu gerade bearbeiteten Paket
	if (packet == f->owner)
	{
		// Paket in Bearbeitung, Flit bekannt machen
		for(unsigned int i=0; i<customers.size(); i++)
		{
			// nur registrierte Empfaenger und 
			if (customers[i].valid)
			{	
				CustomerInfo& c_info = customers[i];
			
				// Paket-Klon fuer Empfaenger aktualisieren, d.h. angekommenes
				// Flit ersetzt Flit an gleicher Stelle in Paket-Klon, so dass
				// Paket-Klon keine Flits enthaelt, die in vorhergehender Stufe
				// geloescht wurden.
				(*c_info.packet)[f->Offset()] = f;
			
				// Empfaenger hat bereits alle vorhandenen Flits abgeholt
				if (c_info.offset==flitlist.end())
				{
					// Annahme: Neues Flit kein Head
					assert(!f->head());

					// Setzen auf erstes und einziges Element
					c_info.offset--;
				}
			}
		}
	}
}

void RegisterContainer::eraseFlit(Flit* f)
{

	// Annahme: Flitliste ist nicht leer
        assert(!flitlist.empty());

	// Infos zu vordersten Flit
	FlitInfo& info = flitlist.front();

	// Annahme: Paket/Flit registriert und Loeschen zulaessig
	assert(packet != NULL);
        assert(f!=NULL);
	assert(f->owner==packet);
        assert(info.flit==f);
	
	// Speicher fuer Bitmaske offener Ziele freigeben
	delete info.remain;	
	// Infos zu vordersten Flit entfernen
	flitlist.pop_front();

	// Ende eines Paketes
	if(f->tail())
	{
		// Annahme: keine weiteren Empaenger fuer Paket registriert
		assert(!counter.registered);
				
		packet->release();
		packet=NULL;
			
		// Naechstes Paket, falls vorhanden
     		if (flitlist.size() > 0)
			packet = flitlist.front().flit->owner;
	}	
}

const Flit* RegisterContainer::fetchableFlit (Node* request) const
{	
	// Annahme: Empfaenger gueltig und Puffer (Target)
	assert(request != NULL && (request->rtti()<=Component::TARGET));
	// Annahme: Empfaenger hat eingehende Leitung
	assert(request->In().size());

        if (flitlist.empty())
                return NULL;

	// Annahme: Flit(s) registriert, also auch Paket vorhanden!
	assert(packet);

	unsigned int index = request->In().front()->getOutIndex();
	
	if (customers.size() > index) 
	{
		const CustomerInfo& c_info = customers[index];
		
		// Knoten ist registriert, hat in diesem Takt noch kein Flit abgeholt
		if( c_info.valid && !c_info.fetched.valid() && !c_info.rejected.valid() 
		// und hat Ende des akuellen Paketes oder der Flitliste noch nicht erreicht
		&& (c_info.offset != flitlist.end()) && ((*c_info.offset).flit->owner == packet))
		{		
			// Flit, welches fetchFlit liefern wuerde 
			return c_info.packet->at((*c_info.offset).flit->Offset());
		}
	}
	
	// kein Flit verfuegbar
	return NULL;
}

Flit* RegisterContainer::fetchFlit (Node* customer)
{
	// Annahme: Empfaenger gueltig und Puffer (Target)
	assert(customer != NULL && customer->rtti() <= Component::TARGET);
	// Annahme: Empfaenger hat eingehende Leitung
	assert(customer->In().size());

	// eindeutiger Index des Empfaengerknotens
	unsigned int index=customer->In().front()->getOutIndex();

	// Verschiedene Zustandsannahmen zu Verwaltungsstrukuren
	assert(!flitlist.empty());
	assert(packet != NULL);
	assert(customers.size() > index);
	assert(customers[index].offset != flitlist.end());
	assert(customers[index].valid && customers[index].packet);
	assert(!customers[index].fetched.valid());
	assert(!customers[index].rejected.valid());
     
	// Informationen zu Flit und Empfaenger
	CustomerInfo& c_info = customers[index];
	FlitInfo& f_info = *c_info.offset;
	c_info.offset++;
	
	// Annahmen: Richtiges Flit und noch Ziele offen 
	assert(f_info.flit->owner == packet);
	assert(f_info.remain && !f_info.remain->empty());
	assert(c_info.reachable && !c_info.reachable->empty());
	
	// Flit-Klon fuer Weitervermittlung
	Flit** clone = &c_info.packet->at(f_info.flit->Offset());

	// Flitaddresse nur kopiert, Flit nicht geklont
	if ((*clone)->owner != c_info.packet)
	{		
		// Annahme: Pointer auf Flist in Liste
		assert((*clone) == f_info.flit);
		
		// Flit klonen	
		(*clone) = f_info.flit->clone();
		(*clone)->setOwner(c_info.packet);
	}
		
	// Bitmaske des kopierten Paketes zur Weitervermittlung aendern, 
	// sofern Head-Flit; Paket kann alle Ziele erreichten, die Abnehmer 
	// und Paket gemeinsam in ihren Bitmasken tragen
	if ((*clone)->head())
	{		
		// Gemeinsame Bits in Bitmasken von Flit und Abnehmer
		Bitmask common = (*c_info.reachable) & (*f_info.remain);

		// Annahme: Gemeinsame Bits vorhanden
		assert(!common.empty());
			
		// Paket bekommt neue, ueber Empfaenger erreichbare Ziele
		(*clone)->owner->setBitmask(common);
	
		// Bei Abholung weiterer Flits sollen stets die gleichen 
		// Bits abgezogen werden
		(*c_info.reachable) = common;
	}
		
	// Bitmaske des originalen Flits aendern; alle Ziele die
	// ueber Abnehmer bedient werden, koennen aus Bitmaske des
	// aktuell vermittelten Flits entfernt werden
	(*f_info.remain)-=(*c_info.reachable);
		
	// Empfaenger markieren
	c_info.fetched.sign();
	
	// Zaehler aktualisieren
	counter.refresh();

	/*std::cerr << "COUNTER (PRE): " 
		<< counter.registered << " : " 
		<< counter.fetched << " : " 
		<< counter.rejected 
		<< std::endl;*/
	
	/*std::cerr << "<" << Tick() << "> " << "FETCHED: " << customer->toString() << std::endl;*/

	// Zaehler setzen
	if((*clone)->tail())
	{
		// Tail abgeholt, Empfaenger-Knoten endgueltig fertig
		counter.registered--;		
		
		// Paket freigeben
		c_info.packet->dereference();
		// Registrierung aufheben
		c_info.reset();
	}
	else
		// Data/Head abgeholt, Knoten fuer diesen Takt fertig
		counter.fetched++;
	
	// Signal an Puffer, dass Flit vermittelt, so dass Puffer bei vollstaendiger
	// Vermittlung BufferQueue::pop aufruft und damit RegisterContainer::eraseFlit
	f_info.flit->location.where->flitTransmitted(f_info.flit, f_info.remain->empty());

	/*std::cerr << "COUNTER (POST): " 
	<< counter.registered << " : " 
	<< counter.fetched << " : " 
	<< counter.rejected 
	<< std::endl;*/

	return *clone;
}


void RegisterContainer::rejectFlit( Node* customer )
{	
	// Annahme: Empfaenger gueltig und Puffer (Target)
	assert(customer != NULL && customer->rtti() <= Component::TARGET);
	// Annahme: Empfaenger hat eingehende Leitung
	assert(customer->In().size());

	// eindeutiger Index des Empfaengerknotens
	unsigned int index=customer->In().front()->getOutIndex();
		
	// Annahme: Knoten ist registriert
	assert(customers.size() > index && customers[index].valid);
	// Annahme: Flit verfuegbar
	assert(customers[index].offset != flitlist.end());
	
	CustomerInfo& c_info = customers[index];
	FlitInfo& f_info = *c_info.offset;
	
	// Annahme: Knoten hat weder Flit abgeholt noch zurueckgewiesen
	assert(!c_info.fetched.valid() && !c_info.rejected.valid());
	
	counter.refresh();
	
	/*std::cerr << "COUNTER (PRE): " 
		<< counter.registered << " : " 
		<< counter.fetched << " : " 
		<< counter.rejected 
		<< std::endl;*/

	/*std::cerr << "<" << Tick() << "> " << "REJECTED: " << customer->toString() << std::endl;*/
	
	counter.rejected++;
	
	/*std::cerr << "COUNTER (POST): " 
		<< counter.registered << " : " 
		<< counter.fetched << " : " 
		<< counter.rejected 
		<< std::endl;*/
		
	// Flit-Klon fuer Weitervermittlung
	Flit** clone = &c_info.packet->at(f_info.flit->Offset());
	
	// Flitaddresse nur kopiert, Flit noch nicht geklont
	if ((*clone)->owner != c_info.packet)
	{
		// Annahme: Pointer auf Flist in Liste
		assert((*clone) == f_info.flit);
		
		// Flit klonen	
		(*clone) = f_info.flit->clone();
		(*clone)->setOwner(c_info.packet);
	}
	
	// Empfaenger und Flit markieren
	(*clone)->rejected();
	c_info.rejected.sign();
}


void RegisterContainer::registerCustomer( Node* node, Bitmask* targets )
{
	// Annahme: Empfaenger ist Puffer (Target)
	assert(node->rtti() <= Component::TARGET);
	// Annahme: Empfaenger hat eingehende Leitung
	assert(node->In().size());
	
	// Index des Knotens ist der Ausgangsindex der eingehenden
	// Leitung. Leitung kann bei Knoten dieses Registers beginnen
	// oder an einem Router. Im zweiten Fall ist der Index der
	// eindeutige Index eines Routerausgangs.
	unsigned int index = node->In().front()->getOutIndex();

	// Liste der Empfaenger ggf. erweitern
	if(customers.size() <= index)
		customers.resize(index+1);
	
	// Annahme: Erreibare Ziele wurden spezifiziert
	assert(targets && !targets->empty());
	
	// Knoten bereits registriert
	if(customers[index].valid)
	{
		delete targets;
		return;
	}
	
	// Annahme: Ein Paket ist verfuegbar
	assert(packet != NULL);
	
	// Annahme: Struktur zu Paket-Klon neu oder aufgeraeumt
	assert(customers[index].packet==NULL);
	assert(customers[index].reachable==NULL);
	assert(customers[index].customer==NULL);

	counter.refresh();

	/*std::cerr << "COUNTER (PRE): " 
		<< counter.registered << " : " 
		<< counter.fetched << " : " 
		<< counter.rejected 
		<< std::endl;*/

	/*std::cerr << "<" << Tick() << "> " << "REGISTERED: " << node->toString() << std::endl;*/
			
	counter.registered++;
	
	/*std::cerr << "COUNTER (POST): " 
		<< counter.registered << " : " 
		<< counter.fetched << " : " 
		<< counter.rejected 
		<< std::endl;*/

	CustomerInfo& c_info=customers[index];
	c_info.packet=packet->clone();
	c_info.packet->reference();
	c_info.offset=flitlist.begin();
	c_info.valid=true;
	c_info.customer=node;
	c_info.reachable=targets;
}

void RegisterContainer::unregisterCustomer( Node* node )
{
	// Annahme: Empfaenger ist Puffer (Target)
	assert(node->rtti() <= Component::TARGET);
	// Annahme: Empfaenger hat eingehende Leitung
	assert(node->In().size());
	
	// Index des Knotens ist der Ausgangsindex der eingehenden
	// Leitung. Leitung kann bei Knoten dieses Registers beginnen
	// oder an einem Router. Im zweiten Fall ist der Index der
	// eindeutige Index eines Routerausgangs.
	unsigned int index = node->In().front()->getOutIndex();

	// Knoten nicht registriert
	if(customers.size() <= index || !customers[index].valid)
		return;

	counter.refresh();
	
	/*std::cerr << "COUNTER (PRE): " 
		<< counter.registered << " : " 
		<< counter.fetched << " : " 
		<< counter.rejected 
		<< std::endl;*/

	/*std::cerr << "<" << Tick() << "> " << "UN-REGISTERED: " << node->toString() << std::endl;*/
	
	// Annahme: Knoten hat noch kein Flit abgeholt
	assert(!customers[index].fetched.valid());
	// Annahme: Mehr Knoten registriert als bearbeitet
	assert(counter.registered>counter.fetched+counter.rejected);
	
	// Anzahl registrierter Knoten anpassen
	counter.registered--;

	// Knoten hat bereits verloren
	if(customers[index].rejected.valid())
		counter.rejected--;
	
	/*std::cerr << "COUNTER (POST): " 
		<< counter.registered << " : " 
		<< counter.fetched << " : " 
		<< counter.rejected 
		<< std::endl;*/
	
	// Annahme: Fuer registrierten Empfaenger wurde Paket-Klon angelegt
	assert(customers[index].packet);
	
	// Registrierung aufheben
	customers[index].packet->release();
	customers[index].reset();
}
	

const Bitmask* RegisterContainer::validTargets( Node* customer )
{
	unsigned int index = customer->In().front()->getOutIndex();
	
	if(customers.size() > index && customers[index].valid)
	{
		return customers[index].reachable;
	}
	return NULL;
}


const Flit* RegisterContainer::firstFlit() const
{
	if(flitlist.size())
		return flitlist.front().flit;

	return NULL;
}


bool RegisterContainer::finalState()
{
	counter.refresh();

	return  counter.registered == counter.rejected+counter.fetched;
}

void RegisterContainer::Counter::refresh()
{	
	if(refreshed.valid())
		return;
	
	// assert(registered == fetched + rejected);
	// Auskommentiert, weil es nicht immer stimmt fr dynamische Rek.
	
	rejected=0; 
	fetched=0; 
	refreshed.sign(); 
}

void RegisterContainer::Counter::reset()
{	
	rejected=0; 
	fetched=0; 
	registered=0;
}

void RegisterContainer::CustomerInfo::reset() 
{
	valid=false; 
	offset = (std::list<FlitInfo>::iterator)0; 
	customer=NULL; 
	packet=NULL; 
	if(reachable) delete(reachable);
	reachable=NULL; 
}

