#include <routing.h>
#include <wirematrixiterator.h>
#include <wirematrixentry.h>
#include <packet.h>
#include <macros.h>
#include <node.h>
#include <bufferqueue.h>
#include <registercontainer.h>

#include <cassert>

int Routing::precondition(WireMatrixIterator* it) const
{
	WireMatrixEntry& entry = it->value();
		
	// Kein Paket, kein Routing
	if(!entry.test(I_PACKET))
		return false;
	
	// Annahme: Ausgaenge zu einem Eingang werden iteriert
	assert(it->rtti()==WireMatrixIterator::OUTWIRE_IT);
	// Annahme: Wenn Paket angezeigt, dann auch vorhanden
	assert(!entry.test(I_PACKET) || entry.currentPacket());
	// Annahme: Wenn noch Ziele offen, dann Head auf der Line
	assert(!entry.remaining()->empty() || entry.getInWire()->getFlit()->head());
	// Annahme: Wenn Routingentscheidung vorliegt, dann Flit vorhanden
	assert(!entry.test(IO_VALID) || (entry.getInWire()->getFlit(entry.getOutWire()->To())));
	
	return
	// 1. noch keine Routing-Entscheidung	
	!entry.test(IO_VALID)
	// 2. Paket liegt an und noch Ziele offen, die ueber Ausgang erreichbar	
	&& entry.test(I_PACKET) && (!entry.targets() || ((*entry.remaining()) <= (*entry.targets())));
}

void Routing::modify(WireMatrixIterator* it)
{	
	// Matrixeintrag
	WireMatrixEntry& entry = it->value();

	// Maske mit erreichbaren Zielen ueber Ausgang
	Bitmask* valid =  validOutput(entry.currentPacket(), entry.remaining(), entry.getOutWire());
	
	if(valid)
	{		
		_PACKETMOVE("Possible route:", entry.currentPacket()->toString(),
				"From: " << entry.getInWire()->From()->toString() << std::endl
				<< " To: " << entry.getOutWire()->To()->toString());
	
		// Verbindung zulaessig
		entry.set(IO_VALID);
	
		// Moeglichen Empfaenger bei Register des Senders anmelden
		entry.getInWire()->RegisterOnLine()->registerCustomer(entry.getOutWire()->To(), valid);
	}
}

