#ifndef __BITMASKROUTING__
#define __BITMASKROUTING__

#include <routing.h>

//! Standard-Routingverfahren.
/*!
 * Diese Klasse implementiert das Standard-Routing in CINSim. Fuer die 
 * Routingentscheidungen erfolgt ein Vergleich der Bitmasken, der zur 
 * Verfuegung stehenden Ausgaenge.
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class BitmaskRouting : public Routing
{
public:

	//! Implementierung der abstrakten Basisklasse Routing.
	/*!
	 * Fuer dieses Routing-Verfahren ist keine Initialisierung
	 * notwendig.
	 * 
	 * @see Routing
	 */
	bool init() { return true; } 
		
protected:

	//! Implementierung der abstrakten Basisklasse Routing.
	/*!
	 * Diese Methode implementiert eines Test, der festlegt, ob 
	 * ein Paket auf dem Weg zu seinem Ziel den uebergebenen Ausgang 
	 * eines Routers benutzen kann. Als Standard dient ein Test 
	 * einer Bitmaske.
	 * 
	 * @param ptr Das anliegende Paket.
	 * @param remaining Offenene Ziele des anliegenden Paketes.
	 * @param output Ein zu untersuchender Ausgang.
	 * @return Nichtleere Bitmaske, falls Paket ueber spezifizierten 
	 * Ausgang zu offenen Zielen versendet werden kann, oder NULL.
	 */
	virtual Bitmask* validOutput(const Packet* ptr, const Bitmask* remaining, const Wire* output);
};

#endif

