#ifndef __SHORTESTPATHROUTING__
#define __SHORTESTPATHROUTING__

#include <routing.h>
#include <utility/bitmask.h>

//! Hilfs-Struktur fuer minimale Weglaengen.
/*!
 * Zur Bestimmung der minimalen Weglaengen muessen initial alle 
 * Weglaengen auf den groessten moeglichen Wert gesetzt werden 
 * (unendlich). Damit dazu nicht die Matrix durchlaufen werden 
 * muss, setzt der Standard-Konstruktur fuer die Matrix-Eintraege
 * die entsprechenden Maxima.
 */
struct Distance
{
	Distance() : value(UINT_MAX) {} // infinite
	unsigned int value;
};

//! Shortest-Path-Routing.
/*!
 * Diese Klasse implementiert ein Routing-Verfahren basierend auf den 
 * kuerzesten Wegen zu einem Ziel. Dabei wird mit globalem Wissen
 * gearbeitet, d.h. es werden bei der Initialisierung die kuerzesten
 * Weglaengen von einem Puffer zu allen Zielen bestimmt. Ein Ausgang
 * ist nur zulaessig, wenn er minimale Weglaenge zum Ziel garantiert.
 * 
 * TODO: Packetgeneratoren einbinden
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class SPRouting : public Routing 
{
public:

	//! Implementierung der abstrakten Basisklasse Routing.
	/*!
	 * Der Aufruf dieser Methode initialisiert eine Liste von
	 * Bitmasken, die zu jedem Puffer im Netz, der am Ausgang eines
	 * Routers liegt, eine Bitmaske mit zulaessigen, d.h. ueber 
	 * minimale Pfade erreichbaren, Zielen bereitstellt.
	 *  
	 * @see Routing
	 */
	bool init();
	
protected:
		
	//! Implementierung der abstrakten Basisklasse Routing.
	/*!
	 * Durch Verwendung der bei der Initialisierung zum betrachteten 
	 * Ausgang generierten Bitmaske, wird die Bedingung minimaler
	 * Weglaengen gewahrt.
	 * 
	 * @see Routing
	 */
	Bitmask* validOutput(const Packet* ptr, const Bitmask* remaining, const Wire* output);

private:

	//! Vektor mit Bitmasken zu Puffern des Netzes.
	std::vector<Bitmask> bitmasks;
};
	
#endif

