#include <xyrouting.h>
#include <configuration.h>
#include <targetbuffer.h>
#include <router.h>
#include <utility/coordinate.h> 
#include <macros.h>

#include <cassert>

REGISTER_ELEMENT(XYRouting, "StandardXY")

bool XYRouting::init()
{	
	const ComponentVector& router = Components(Component::ROUTER);
	const ComponentVector& buffers = Components(Component::BUFFER);
	const ComponentVector& targets = Components(Component::TARGET);
	
	// Eine Bitmaske fuer jeden Puffer
	bitmasks.resize(buffers.size());
	
	// Alle Router
	for(unsigned int r=0; r<router.size(); r++)
	{	
		// Koordinaten des untersuchten Routers.
		std::vector<int>& currentpos = ((Node*)router[r])->position;
		
		// Annahme: Position des Routers vorhanden
		assert(currentpos.size() == 2);
					
		// Alle Ausgaenge eines Routers
		for(unsigned int o=0; o<((Node*)router[r])->Out().size(); o++)
		{
			// Nachfolger
			Node* next = ((Node*)router[r])->Out()[o]->To();

			if(next->rtti() == Component::TARGET)
				continue;
			
			if(next->rtti() != Component::BUFFER)
				_ERROR("Expected buffer or target following router " 
						<< router[r]->InternalName() << "!");
			
			// Koordinaten des Nachfolgers
			std::vector<int>& nextpos = next->position;
			
			// Annahme: Position des Knoten am Ausgang vorhanden
			assert(nextpos.size() == 2);

			// Alle Ziele
			for(unsigned int t=0; t<targets.size(); t++)
			{
				// Koordinaten des untersuchten Zieles.
				std::vector<int>& targetpos = ((Node*)targets[t])->position;

				// Annahme: Position des Ziels bekannt
				assert(targetpos.size() == 2);

				// Bedingung: Verbindung zu Ziel ueber anderen Gitterknoten 
				if(!next->getBitmask().test(t) || currentpos==targetpos)
					continue;

				if(westOf(currentpos, targetpos)){
					if(westOf(currentpos, nextpos))
						bitmasks[next->Index()].set(t);
					continue;
				}
				if(eastOf(currentpos, targetpos)){
					if(eastOf(currentpos, nextpos))
						bitmasks[next->Index()].set(t);
					continue;
				}
				if(northOf(currentpos, targetpos)){
					if(northOf(currentpos, nextpos))
						bitmasks[next->Index()].set(t);
					continue;
				}
				if(southOf(currentpos, targetpos)){
					if(southOf(currentpos, nextpos))
						bitmasks[next->Index()].set(t);
					continue;
				}
			}
		}
	}

	return true;
}


Bitmask* XYRouting::validOutput(const Packet* ptr, const Bitmask* remaining, const Wire* outwire)
{		
	const Bitmask *valid = NULL;
	
	if(outwire->To()->rtti() == Component::TARGET)
		valid = &outwire->To()->getBitmask();
	
	else if(outwire->To()->rtti() == Component::BUFFER)
		valid = &bitmasks[outwire->To()->Index()];
		
	else _ERROR("Expected buffer or target here!");
		
	// Test gegen Bitmaske mit zulaessigen Zielen
	if((*remaining) <= (*valid))
	{
		// Annahme: Ueber Knoten sollte Ziel erreichbar sein
		assert(!valid->empty());
		
		return new Bitmask((*remaining) & (*valid));
	}
	return NULL;
}

