#ifndef __XYROUTING__
#define __XYROUTING__

#include <routing.h>
#include <utility/bitmask.h>

#include <vector>

//! XY-Routing fuer Gitternetze.
/*!
 * Diese Klasse implementiert ein XY-Routing fuer Gitternetze. Pakete 
 * werden zuerst auf die korrekte X-Koordinate ihrer Ziele bewegt, dann auf 
 * die Y-Koordinate. Setzt voraus, dass die Netzknoten ueber entsprechende 
 * Positionsinformationen verfuegen. Sind diese nicht vorhanden, so kann 
 * kein Routing stattfinden.
 *
 * Hinweis: Die Gitternetze muessen vollstaendig sein und einer rechteckige
 * Grundform aufweisen.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class XYRouting : public Routing
{
public:

	//! Implementierung der abstrakten Basisklasse Routing.
	/*!
	 * Bei der Initialisierung des XY-Routings wird das zu simulierende
	 * Gitternetz untersucht und ausgehend von den Bitmasken der Puffer 
	 * im Netz, die die theoretisch von einem Puffer erreichbarene Ziele
	 * beschreiben, eingeschraenkte Bitmasken generiert, die die zulaessigen
	 * Ziele zu jedem Puffer aus Sicht eines schaltenden Routers beschreiben.
	 * 
	 * @see Routing
	 */
	bool init();
		
protected:
		
	//! Implementierung der abstrakten Basisklasse Routing.
	/*!
	 * Durch Verwendung der bei der Initialisierung zum betrachteten Ausgang
	 * generierten Bitmaske, wird die XY-Bedingung gewahrt.
	 * 
	 * @see Routing
	 */
	Bitmask* validOutput (const Packet* ptr, const Bitmask* remaining, const Wire* output);

private:

	//! Bitmasken zulaessiger Ziele fuer alle Puffer im Netz.
	std::vector<Bitmask> bitmasks;
};

#endif
