#include <simulation.h>
#include <xmlsimulation.h>
#include <configuration.h>
#include <signature.h>
#include <packet.h>

#include <dlhandler.h>
#include <bufferqueue.h>
#include <packetforwarding.h>
#include <routing.h>

#include <macros.h>

#include <time.h>
#include <math.h>

#include <sstream>
#include <iostream>
#include <cfloat>

#include <environment/analysis.h>

Simulation* Simulation::_instance = new XmlSimulation();

unsigned int Signature::_reference = 1;


using namespace std;


Simulation::Simulation() : XmlLoad()
{
	_activeConfig = NULL;
	_initialConfig = NULL;
	
	_globalTick = 0;
	_stateTicks = 0;

	_state = NOSTATE;
	_type = NOTYPE;

	_analysisStart = 0;
	_analysisEnd= UINT_MAX;
}

Simulation::~Simulation()
{
	for(unsigned int i=0; i<_configurations.size(); i++)
	{
		delete _configurations[i];
	}

	_configurations.clear();
}

void Simulation::Info()
{
	//
	// Informationen zu (geparster) Simulation ausgeben
	//
	// - obere Schranken fuer Leerlaufzeiten
	// - Wahrscheinlichkeiten fuer Paketverteilungen
	//

	_ERROR("No implemented yet!");
}


void Simulation::start()
{	
	assert(_initialConfig);
		
	bool *noWarningShown = new bool[_configurations.size()]; // eigentlich char[]

	for(unsigned int i=0; i<_configurations.size(); i++) noWarningShown[i]=true;


// neu 

	bool noUnsolvableDLShown = true;
	bool noCorpseWarnShown = true;
	bool noDeletePacketShown = true;
	bool noOFWarnShown = true;
	int simRun = 1;

	_handler = new DLHandler();

// ---

	_state = NORMAL;
	_activeConfig = _initialConfig;
	_handler->setConfig(_activeConfig);

	_globalTick = 0;
	_stateTicks = _activeConfig->OperationPhase() - 1;

	while(!Analysis::Finished()) // bis Genauigkeit der Messung(en) erreicht
	{
		assert(_type < NOTYPE);
		assert(_state < NOSTATE);

		// Wenn NORMAL und Netz als Kritisch erkannt, dann analysiere auf Deadlocks
		if(_state == NORMAL && _handler->Critical()) 
		{
			// Abhngigkeitstabellen aufstellen
			_handler->setTables();
			// Abhngigkeitsbume aufstellen
			_handler->setTrees();
			// Wenn keine Lsung aufzufinden, dann Warnung ausgeben
			if(!_handler->Solution() && noUnsolvableDLShown)
			{
				_WARNING("In configuration " << _activeConfig->Index() 
					<< " first unsolvable deadlock situation occurrance after " << simRun << " simulation runs. "
					<< "(This warning will only appear once)");
				noUnsolvableDLShown = false;
			}
			// Setze die Flags der Puffer so, dass eine Auflsung resultiert
			_handler->forceSwitching();
		}
		
		
		ComponentVector currentBuffers = _activeConfig->Components(Component::BUFFER);
		ComponentVector currentTargets = _activeConfig->Components(Component::TARGET);
		
		if(_state == NORMAL || _state == PREPARE)
			_activeConfig->advance();

		// schneide alle Puffer zurck, wenn cut-signiert
		// setze alle MaeanderPuffer zurck, wenn solve_maeander signiert
		// Bei dieser Gelegenheit prfe auch, ob ein Puffer berfllt ist.
		for(ComponentVector::iterator itp = currentBuffers.begin();itp!=currentBuffers.end();itp++)
		{
			Buffer* itpBuffer = (Buffer*)(*itp);
			if(itpBuffer->BQ->cut.valid() && itpBuffer->BQ->infinite)
				itpBuffer->BQ->infinite = false;
			
			if(itpBuffer->solve_maeander.valid())
				itpBuffer->maeander = false;
			
			if(itpBuffer->BQ->queueSpace() < itpBuffer->BQ->spaceInUse() && noOFWarnShown)
			{
				_WARNING("Buffer was forced to overflow, after " 
					<< simRun << " simulation runs. "
					<< "(This warning will only appear once)");
				noOFWarnShown = false;
			}

		}		

		Signature::InvalidateAll();
				
		if(_globalTick == _analysisEnd)
		{
			_activeConfig->reset();
			_activeConfig = _initialConfig;
			
			_state = NORMAL;
			_globalTick = 0;
			_stateTicks = _activeConfig->OperationPhase() - 1;
			
			// assert(!Packet::Existing());
				
			// schneide alle verbleibenden infinite-Puffer zurck
			// setze alle verbleibenden MaeanderPuffer zurck
			bool remainingMarks = false;
			for(ComponentVector::iterator itp = currentBuffers.begin();itp!=currentBuffers.end();itp++)
			{
				Buffer* itpBuffer = (Buffer*)(*itp);
				if(itpBuffer->BQ->infinite)
				{
					remainingMarks = true;
					itpBuffer->BQ->infinite = false;
				}
				if(itpBuffer->maeander)
				{
					remainingMarks = true;
					itpBuffer->maeander = false;
				}
			}	
						
			if(remainingMarks && noCorpseWarnShown)
			{
				/* _WARNING("First corpse found, i.e. unroutable packet remained, after " 
					<< simRun << " simulation runs. "
					<< "(This warning will only appear once)"); */
				noCorpseWarnShown = false;
			}
			// Zhle Simulationslauf eins weiter
			simRun++;
			
			continue;
		}
		
		if(_state == NORMAL && !_stateTicks)
		{
			_state = PREPARE;
;
			_stateTicks = _activeConfig->DrainPhase();

			_DEBUG("End of operation phase for configuration " 
					<< _activeConfig->Index() << ", " 
					<< Packet::Existing() << " packets seen");
		}
				
		if(_state == PREPARE && !_stateTicks)
		{
			_state = SWITCH;
			_stateTicks = _activeConfig->ReconfigurationPhase();
			
			// Wenn statisch, dann ist ein nicht-leeres Netz nach der Vorbereitungsphase ein Fehler
			if(Packet::Existing() && noWarningShown[_activeConfig->Index()] && !_activeConfig->Dynamic())
			{
				_WARNING("Configuration " << _activeConfig->Index() 
						<< " was not empty at the end of its packet drain phase! "
						<< " (This warning will appear only once)" );
			
				noWarningShown[_activeConfig->Index()] = false;
			}
			
			_DEBUG("End of packet drain phase for configuration " 
					<< _activeConfig->Index() << ", " 
					<< Packet::Existing() << " packets seen");
		}

		if(_state == SWITCH && !_stateTicks)
		{
			assert(_activeConfig);
			assert(_activeConfig->Next());
			assert(_activeConfig->Next()->Index() < _configurations.size());
		
			_DEBUG("End of reconfiguration phase for configuration " 
					<< _activeConfig->Index() << ", " 
					<< Packet::Existing() << " packets seen");
			
			Configuration* _nextConfig = _configurations[_activeConfig->Next()->Index()];
			
			ComponentVector oldBuffers = _activeConfig->Components(Component::BUFFER);
			ComponentVector oldTargets = _activeConfig->Components(Component::TARGET);
			ComponentVector newBuffers = _nextConfig->Components(Component::BUFFER);
			ComponentVector newTargets = _nextConfig->Components(Component::TARGET);

			int suxIndex = 0;
						
			//
			// kopiert die Pakete in die neue Konfiguration (nur wenn dynamic)
			// 
			
			if(_activeConfig->Dynamic())
			{
				// Gehe durch alle Buffer der alten Konfiguration
				for(ComponentVector::iterator it1 = oldBuffers.begin();it1!=oldBuffers.end();it1++)
				{
					Buffer* it1Buffer = (Buffer*)(*it1);
					// lsche alle vordersten Flits des Buffers it1, die keine Heads sind
					while(!it1Buffer->BQ->empty() && !it1Buffer->BQ->front()->head())
					{
						it1Buffer->BQ->raw_pop();
					}
					
					// ermittle allflligen Nachfolger von it1Buffer
					suxIndex = _activeConfig->_nextBuffer[it1Buffer->Index()];	

					// wenn der Buffer it1 einen Nachfolger hat (nur wenn dynamisch...)
					if(suxIndex>-1 /*&& false*/) 
					{					
						// solange der Buffer noch Flits enthlt
						while(!it1Buffer->BQ->empty())
						{
							// nenne Owner myP von Front-Flit
							const Packet* myP = it1Buffer->BQ->front()->Owner();

							// gib Bitmask bOld von myP
							const Bitmask* bOld = &myP->getBitmask();
						
							// klone myP zu myPclone
							Packet* myPclone = myP->clone(true);
						
							// kopiere Bitmask vom Ownerpaket und lsche alle bits
							Bitmask* bNew = new Bitmask(*bOld);
							bNew->clear();

							// Gehe durch jedes Bitfeld der alten Maske bOld
							for(unsigned int k=0;k<oldTargets.size();k++)
							{
								// Wenn Bitfeld fr altes Target k einen Nachfolger hat,
								// und das Paket dort hin will, setze bit fr Nachfolger.
								if(!(_activeConfig->_nextTarget[k]<0)&&(bOld->test(k)))
									bNew->set(_activeConfig->_nextTarget[k]);	
							}
					
							// gib dem kopierten Paket die Bitmaske bNew
						
							myPclone->setBitmask(*bNew);
						
							Node::Filter* nf = new TriggerNode::Filter();
							NodeVector nv = ((Buffer*)newBuffers[suxIndex])->nextNodes(nf);
							Bitmask valid;
						
							for(unsigned int i=0; i<nv.size(); i++)
							{
								// Wenn Weg fr myPclone ber nv[i] erlaubt
								Bitmask* bValid = 
									_nextConfig->Coroutine()->routing()->validOutput(
										myPclone,
										bNew,
										nv[i]->In()[0]);
							
								if(bValid)							
									// merke (oder-gleich) Weg vor.
									valid |= *bValid;	
				
							}
						
							// setze jedes Flit dieses Pakets in den neuen Buffer ein
							// und entferne das entsprechende Original im alten Buffer						
							// Paket nur kopieren, wenn Routing im neuen Netz noch mglich
							if(!valid.empty())
							{
								// Wo du hin willst UND kannst, gehe hin.
								*bNew &= valid;

								assert(!bNew->empty());
							
								myPclone->setBitmask(*bNew);
								
								for(unsigned int i=0;i<myPclone->size();i++)
								{
									// ((Buffer*)newBuffers[suxIndex])->flitReceived();
									myPclone->at(i)->buffered((Buffer*)newBuffers[suxIndex], 
											myPclone->at(i)->Location().since);
														
									((Buffer*)newBuffers[suxIndex])->BQ->push(myPclone->at(i));

									it1Buffer->BQ->raw_pop();
								}
							}
							else
							{					
								for(unsigned int i=0;i<myPclone->size();i++)
									it1Buffer->BQ->raw_pop();
								myPclone->release();
								if(noDeletePacketShown)
								{
									_WARNING("Configuring " << _activeConfig->Index() 
										<< " Caused a packet to be deleted after " 
										<< simRun << " simulation runs. "
										<< "(This warning will only appear once)");
									noDeletePacketShown = false;
								}
							}
						
							delete bNew;
						}
					}
				}
			}
			
			_activeConfig->reset();
			_activeConfig = _nextConfig;
			
			assert(_activeConfig);
			
			_state = NORMAL;
			_stateTicks = _activeConfig->OperationPhase();

			// _handler soll hier die neue Config erhalten.
			// Dabei wird automatisch critical=true
			_handler->setConfig(_activeConfig);
			
		}

		assert(_stateTicks > 0);
		
		_globalTick++;
		_stateTicks--;
	}
	
	// fuer eventuellen Neustart
	_activeConfig->reset();
}


void Simulation::reload()
{
	assert(Packet::Existing() == 0);
	
	for(unsigned int i=0; i < _configurations.size(); i++)
	{
		_activeConfig = _configurations[i];
			
		_activeConfig->init();
	}
}	

/** The methode calcurate a packetlife maximal long in one net (Bufferqueue->maxSpace)
 *  her calcurate only the buffer, not the generators and the targets
 */
/*
unsigned int Simulation::getCleanTime()
{

        unsigned int maxSpaceSum=0;

        for (unsigned int i=packetgenerator.size(); i<(buffer.size()-targets.size()); i++)
                maxSpaceSum = maxSpaceSum + buffer[i]->BQ->spaceInUse() + buffer[i]->BQ->freeSpace();
        return maxSpaceSum;
}
*/

