#include <switching.h>
#include <arbiter.h>
#include <wirematrix.h>
#include <wirematrixiterator.h>
#include <wirematrixentry.h>
#include <flit.h>
#include <node.h>
#include <packet.h>
#include <bufferqueue.h>
#include <registercontainer.h>
#include <macros.h>

#include <assert.h>


int Switching::precondition(WireMatrixIterator* it) const
{
	WireMatrixEntry& entry = it->value();

	// Entweder Eingang wird als bearbeitet markiert und dazu 
	// ueber seine Ausgaenge iteriert...
	return (it->rtti()==WireMatrixIterator::OUTWIRE_IT) ||	
	// ...oder fuer noch nicht fertig bearbeiteten Ausgang soll 
	// Scheduling in Schaltung uebertragen werden
	(entry.test(O_SCHEDULED) && !entry.test(O_FINISHED));
}

void Switching::modify(WireMatrixIterator* it)
{	
	WireMatrixEntry& entry = it->value();

	if(it->rtti() == WireMatrixIterator::OUTWIRE_IT)
	{	
		entry.set(I_FINISHED);

		// Redundanter, nicht vermittelter Ausgang 
		if(entry.test(IO_VALID) && !entry.test(IO_WON))
		{
			// Annahme: Register auf der Line 
			assert(entry.getInWire()->RegisterOnLine());
		
			// Ausgang wird nicht mehr beruecksichtigt
			entry.getInWire()->RegisterOnLine()->unregisterCustomer(entry.getOutWire()->To());
			entry.reset(IO_VALID);
		}		
		return;
	}
	
	WireMatrixIterator* rowit = it->crossingIterator();

	// Eintrag fuer Ausgang markieren
	entry.set(O_FINISHED);
	entry.set(O_BUSY);

	if(!entry.test(IO_VALID))
		return;
	
	// Eingang ist fertig und kann geschaltet werden
	if(entry.test(IO_WON))
	{			
		entry.set(IO_SWITCHED);
		entry.set(I_BUSY);
		
		// Annahme: Schnittmenge zwischen offenen und erreichbaren Zielen
		assert(entry.targets() && ((*entry.remaining()) <= (*entry.targets())));
		
		// Erreichbare/Zulaessige Ziele abziehen
		(*entry.remaining()) -= (*entry.targets());

		if(entry.remaining()->empty())
		{
			// Markierung des betrachteten Einganges, Entfernen
			// redundanter Ausgaenge
			rowit->setAll(this);
		}
			
		_PACKETMOVE(entry.getOutWire()->From()->toString() << " switched:",
				entry.getInWire()->getFlit(entry.getOutWire()->To())->toString(),
				"From: " << entry.getInWire()->From()->toString() << std::endl
				<< " To: " << entry.getOutWire()->To()->toString());

		// Schatung herstellen
		entry.getOutWire()->connectRegister(entry.getInWire()->RegisterOnLine());

		return;
	}
}

WireMatrixModifier* Switching::clear() const
{
	return (WireMatrixModifier*)&cs_modifier;
}


void ClearSwitching::modify(WireMatrixIterator* it)
{
	// Annahme: Eingaenge zu einem Ausgang werden iteriert
	assert(it->rtti()==WireMatrixIterator::INWIRE_IT);
	
	WireMatrixEntry& entry = it->value();

	// Ausgang freigeben
	entry.reset(O_BUSY);
	
	// geschalteter Eingang
	if(entry.test(IO_WON))
	{			
		assert(entry.test(IO_VALID));		
		assert(entry.test(IO_SWITCHED));
		assert(entry.test(I_BUSY));
		assert(entry.getOutWire()->To()->rtti() <= Component::TARGET);
		assert(entry.getOutWire()->getFlit(entry.getOutWire()->To()) == NULL);
			
		// Reservierung aufheben
		entry.reset(I_BUSY);
		
		// Leitung freigeben
		entry.getOutWire()->unconnectRegister();
	}
}

