#include <targetbuffer.h>
#include <bufferqueue.h>
#include <macros.h>
#include <configuration.h>
#include <flit.h>
#include <packet.h>
#include <utility/xmlattribute.h>

#include <cassert>

REGISTER_ELEMENT(TargetBuffer, "targetbuffer")


bool TargetBuffer::init(const DOMElement* scope)
{	
	if(scope) setScope(scope);
	assert(Scope()); 
	
	if(!BQ) 
		BQ = new BufferQueue(UINT_MAX);

	return true;
}

double TargetBuffer::Measure(TypeInfo type)
{
	switch (type) 
	{
	// Verzoegerung im Netz (Paket)
	case eDelay:
	case ePrioriDelay:
		assert(_delay.valid());
		return _delay.value;
		break;
	// Latenz im Netz (Paket)
	case eLatency:
	case ePrioriLatency:
		assert(_latency.valid());
		return _latency.value;
		break;
	// Durchsatz (Flit)
	case eTThroughput:
	case ePrioriTThroughput:
		return _throughput.value;
		break;
	default:
		_ERROR("Unexpected measure type: " << type);
	}
	return 0.0;
}

void TargetBuffer::takeMeasure(const Description& descr)
{
	switch (descr.type) 
	{
	// Delay im Netz (Paket)
	case ePrioriDelay:	
		if (_delay.priority != descr.priority) break;
	case eDelay:
		if (_delay.valid())
			Analysable::takeMeasure(descr);
		break;
		
	// Latenz im Netz (Paket)
	case ePrioriLatency:
		if (_latency.priority != descr.priority) break;
	case eLatency:
		if (_latency.valid())
			Analysable::takeMeasure(descr);
		break;

	// Durchsatz (Flit)
	case ePrioriTThroughput:
		if (_throughput.priority != descr.priority) break;
	case eTThroughput:
		if(!_throughput.valid()) _throughput.value = 0.0;
			Analysable::takeMeasure(descr);
		break;
		
	// Keine Messung
	default:
		break;
	}
}

const Bitmask& TargetBuffer::generateBitmask()
{
	// Annahme: Index im Bereich existierender Ziele
	assert(Index() < Components(rtti()).size());
	
	_bitmask.set(Index());

	return _bitmask;
}

void TargetBuffer::reset()
{	
	assert(BQ->spaceInUse()==0);
}

void TargetBuffer::flitReceived(Flit* flit)
{	
	// Paket zu empfangenen Flit
	Packet *packet = flit->owner;
	
	// Test auf korrektes Ziel
	if(!packet->getBitmask().test(Index()))
		_ERROR(flit->toString() << ", arrived at wrong target (" << Index() << ")");

	// Messungen aktualisieren
	_throughput.priority = packet->priority;
	_throughput.value = 1.0;
	_throughput.sign();
	
	if(flit->head())
	{
		_delay.priority = packet->priority;
		_delay.value = Tick()-packet->createTick;
		_delay.sign();
	}
	if(flit->tail())
	{
		_latency.priority = packet->priority; 
		_latency.value = Tick()-packet->createTick;
		_latency.sign();

		packet->release();
	}
}

