#ifndef __TEST_REGISTERCONTAINER_H__
#define __TEST_REGISTERCONTAINER_H__

#include <cppunit/extensions/HelperMacros.h>

class Wire;
class Node;
class Packet;
class Flit;
class RegisterContainer;

using namespace CppUnit;

/*!
 * Unit-Test(s) fuer Klasse RegisterContainer.
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Test_RegisterContainer : public TestFixture
{	
CPPUNIT_TEST_SUITE( Test_RegisterContainer );
CPPUNIT_TEST( registration_test );
CPPUNIT_TEST( storing_test );
CPPUNIT_TEST( receiving_test );
CPPUNIT_TEST_SUITE_END();
	
public:

	/*!
	 * Implementierung der Klasse TestFixture.
	 * 
	 * @see TestFixture
	 */
	void setUp();
		
	/*! 
	 * Implementierung der Klasse TestFixture.
	 * 
	 * @see TestFixture
	 */
	void tearDown();
	
	//! Test der Registrierung neuer Empfaenger.
	/*!
	 * Bevor ein Paket aus einem RegisterContainer empfangen werden kann
	 * muessen sich moegliche Empfaenger registrieren. Wird eine 
	 * Registrierung aufehoben, so koennen auch keine Flits von Paketen 
	 * mehr empfangen werden.
	 */
	void registration_test();
	
	//! Test der Registrierung von Paketen.
	/*!
	 * Bevor Flits von Paketen aus der Queue eines Puffers weitervermittelt 
	 * werden koennen muessen sie beim RegisterContainer registriert werden.
	 * Wurde ein Flit von allen registrierten Empfaengern abgeholt, so wird
	 * die Registrierung aufgehoben.
	 */
	void storing_test();

	//! Test 
	/*!
	 * Die registrierten Empfaenger fuer Pakete aus einem RegisterContainer
	 * koennen nacheinander auf die Flits eines Paketes zugreifen, wobei
	 * keine Synchronisation unter den Empfaenger erfolgt. Wird aber das
	 * Ende eines Paketes erreicht, so kann erst auf das naechste Paket
	 * zugegriffen werden, wenn das aktuelle Paket komplett an alle
	 * Empfaenger versendet wurde.
	 */
	void receiving_test();

private:
	
	Packet *packet1, *packet2;
	RegisterContainer *container0, *container1, *container2;
	Node *buffer0, *buffer1, *buffer2, *router0;
	Wire *wire0; // buffer0->router0;
	Wire *wire1, *wire2; //router0->buffer_i

	Flit* test_flit;
	const Flit* const_test_flit;
	const Packet* test_packet;};


#endif
