#ifndef __TEST_TRIGGERNODE_H__
#define __TEST_TRIGGERNODE_H__

#include <cppunit/extensions/HelperMacros.h>

class TriggerNode;
class SignedTNodeList;

using namespace CppUnit;

/*!
 * Unit-Test(s) fuer Klasse TriggerNode.
 *
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Test_TriggerNode : public TestFixture
{	
CPPUNIT_TEST_SUITE( Test_TriggerNode );
CPPUNIT_TEST( init_check );
CPPUNIT_TEST( construction_check );
CPPUNIT_TEST( signing_check );
CPPUNIT_TEST_SUITE_END();
	
public:

	/*! 
	 * Implementierung der Klasse TestFixture.
	 * 
	 * @see TestFixture
	 */
	void setUp();
	
	/*! 
	 * Implementierung der Klasse TestFixture.
	 * 
	 * @see TestFixture
	 */
	void tearDown();

	//! Test auf korrekte Initialsierung.
	/*!
	 * Eine Instanz der Klasse TriggerNode ist initial nicht signiert
	 * und hat weder Vorgaenger- noch Nachfolgeknoten.
	 */
	void init_check();
	
	//! Test der Baumstruktur.
	/*!
	 * Instanzen der Klassen TriggerNodes koennen in einer Baumstruktur
	 * organisiert werden. Aufbau der Struktur hat keinen Einfluss auf
	 * Signaturen.
	 */
	void construction_check();
	
	//! Test der Signaturen.
	/*!
	 * Die Coroutine signiert TriggerNodes entsprechend ihrer Zustaende
	 * in jedem Takt. Das kann direkt erfolgen oder durch Hinzufuegen zu
	 * Listen signierter Knoten.
	 */
	void signing_check();

private:
	
	//! Testknoten.
	TriggerNode *tnode0, *tnode1, *tnode2;

	//! Listen signierter Knoten.
	SignedTNodeList* notsigned;
	SignedTNodeList* sent;
};


#endif
