#include <utility/configdirectives.h>
#include <macros.h>

#include <iostream>
#include <iomanip>

using namespace std;


ConfigDirectives::ConfigDirectives()
{
	directives.clear();
}
ConfigDirectives::~ConfigDirectives()
{
	directives.clear();
}

ConfigDirectives::ConfigDirectives(const ConfigDirectives& rhs) 
{
	(*this) = rhs;
}

ConfigDirectives& ConfigDirectives::operator=(const ConfigDirectives& rhs) 
{
	if (this != &rhs) 
	{
		directives.clear();

		Directives::const_iterator it = rhs.directives.begin();
		for (; it != rhs.directives.end(); it++)
			directives[it->first] = it->second;
	}
	return *this;
}

ConfigDirectives & ConfigDirectives::operator<<=(const ConfigDirectives & rhs) 
{
	if (this != &rhs) 
	{
		Directives::const_iterator it = rhs.directives.begin();
		for (; it != rhs.directives.end(); it++) 
			directives[it->first] = it->second;
	}
	return *this;
}

ConfigDirectives & ConfigDirectives::operator+=(const ConfigDirectives & rhs) 
{
	if (this != &rhs) 
	{
		Directives::const_iterator it = rhs.directives.begin();
		for (; it != rhs.directives.end(); it++)
			if(!isSet(it->first))
				directives[it->first] = it->second;
	}
	return *this;
}

ValueString ConfigDirectives::getValue(const string& key) const 
{
	Directives::const_iterator it = directives.find(key);
	if ( it == directives.end())
		return ValueString(snull);
	else
		return it->second;
}

bool ConfigDirectives::isSet(const string& key) const 
{
	if ( directives.find(key) != directives.end() )
		return (true);

	return (false);
}

void ConfigDirectives::addValue(string key, ValueString value) 
{
	directives[key] = value;
}

ValueString& ConfigDirectives::operator[](const string& key) 
{
	return (directives[key]);
}

vector<string> ConfigDirectives::getKeys() const 
{
	Directives::const_iterator it = directives.begin();
	vector<string> keys;

	for (;it != directives.end(); it++)
		keys.push_back(it->first);
		
	return keys;
}

bool ConfigDirectives::isEmpty() const 
{
	return directives.empty();
}

void ConfigDirectives::removeValue(const string& key) 
{
	directives.erase(key);
}

void ConfigDirectives::print() const 
{
	this->print(cout);
}

void ConfigDirectives::print(ostream &out) const 
{
	string value;

	// Print out the string directives first.
	for (Directives::const_iterator i = directives.begin(); i != directives.end(); i++ ) 
	{
		value = (string)i->second;
		if (value == snull)
			value = "";
			
		out << left << setw(16) << i->first << " = " << value << endl;
	}
}

/** Clear all of the tables, integer and string */
void ConfigDirectives::clear() 
{  
	directives.clear(); 
}

ostream & operator<<(ostream &out, ConfigDirectives &rhs) 
{
	rhs.print(out);
	return out;
}

