#include <test_bitmask.h>

#include <iostream>

CPPUNIT_TEST_SUITE_REGISTRATION( Test_Bitmask );

#define MAX_TARGET 2047		// > 10
#define RANDOM_BIT ((unsigned int)(drand48() * (MAX_TARGET)))
#define RANDOM_SEED srand48(time (0))

void Test_Bitmask::setUp()
{	
	// Zufallszahlen initialisieren
	RANDOM_SEED;
	
	unsigned int i = 0;
	unsigned int bit = 0;
	
	// neue Bitmasken
	mask0 = new Bitmask;
	mask1 = new Bitmask;
	mask2 = new Bitmask;	
	mask3 = new Bitmask;
	
	// Zufaellig 10 disjunkte Bits generieren, Maske 0 und 1 mit Ueberlappung
	for( ; i < 4; i++ ) 
	{
		do bit = RANDOM_BIT;
		while(mask0->test(bit));
			
		//std::cerr << "Bit " << i << ": " << bit << std::endl;
		
		bits.push_back(bit);
		mask0->set(bit);
	}
	for( ; i < 6; i++ ) 
	{		
		do bit = RANDOM_BIT;
		while(mask0->test(bit));
		
		//std::cerr << "Bit " << i << ": " << bit << std::endl;

		bits.push_back(bit);
		mask0->set(bit);
		mask1->set(bit);
	}
	for( ; i < 10; i++ ) 
	{
		do bit = RANDOM_BIT;
		while(mask0->test(bit) || mask1->test(bit));
		
		//std::cerr << "Bit " << i << ": " << bit << std::endl;

		bits.push_back(bit);
		mask1->set(bit);
	}
}

void Test_Bitmask::tearDown()
{
	if(mask0)
		delete mask0;
	if(mask1)
		delete mask1;
	if(mask2)
		delete mask2;	
	if(mask3)
		delete mask3;

	bits.clear();
}

void Test_Bitmask::init_check()
{
	// noch kein Bits gesetzt 
	CPPUNIT_ASSERT( mask3->empty() );

	mask3->set(17);

	// ein Bit gesetzt 
	CPPUNIT_ASSERT( !mask3->empty() );
}

void Test_Bitmask::size_check()
{
	// Zufaellig Bits setzen und aufheben
	for( unsigned int i = 0; i < 100; i++ )
	{	
		// Zu setzendes Bit
		unsigned int bit = RANDOM_BIT; //rand() >> 16;
		unsigned int fields = 1;

		//std::cerr << "RANDOM: " << bit << std::endl;
		
		CPPUNIT_ASSERT( mask3->fields() == fields );
		CPPUNIT_ASSERT( mask3->empty() );

		mask3->reset(bit);
		
		// kein Einfluss auf alloziierten Speicher oder Bits
		CPPUNIT_ASSERT( mask3->fields() == fields );
		CPPUNIT_ASSERT( mask3->empty() );

		mask3->set(bit);
		fields = mask3->fields();

		// Bit gesetzt
		CPPUNIT_ASSERT( mask3->test(bit) );
		
		mask3->set(bit);
		
		// Bit bleibt gesetzt
		CPPUNIT_ASSERT( mask3->test(bit) );

		fields = mask3->fields();
		mask3->reset(bit);

		// Bit zurueckgesetzt, aber Speicher bleibt alloziiert
		CPPUNIT_ASSERT( !mask3->test(bit));
		CPPUNIT_ASSERT( mask3->fields() == fields );
		CPPUNIT_ASSERT( mask3->empty() );

		// Speicher freigeben
		mask3->clear();
	}
}

void Test_Bitmask::and_check()
{
	unsigned int i = 0;
	
	// Anwendung
	(*mask2) = (*mask0) & (*mask1);
	(*mask1) &= (*mask0);
	
	for( ; i < 4; i++ )
	{
		CPPUNIT_ASSERT( !mask1->test(bits[i]) );
		CPPUNIT_ASSERT( !mask2->test(bits[i]) );
	}	
	for( ; i < 6; i++ )
	{
		CPPUNIT_ASSERT( mask1->test(bits[i]) );
		CPPUNIT_ASSERT( mask2->test(bits[i]) );

		mask1->reset(bits[i]);
		mask2->reset(bits[i]);
	}	
	for( ; i < 10; i++ )
	{
		CPPUNIT_ASSERT( !mask1->test(bits[i]) );
		CPPUNIT_ASSERT( !mask2->test(bits[i]) );
	}	
	
	CPPUNIT_ASSERT( mask1->empty() );
	CPPUNIT_ASSERT( mask2->empty() );
}

void Test_Bitmask::or_check()
{	
	// Anwendung
	(*mask2) = (*mask0) | (*mask1);
	(*mask1) |= (*mask0);
	
	for( unsigned int i = 0; i < 10; i++ )
	{
		CPPUNIT_ASSERT( mask1->test(bits[i]) );
		CPPUNIT_ASSERT( mask2->test(bits[i]) );

		mask1->reset(bits[i]);
		mask2->reset(bits[i]);
	}
	
	CPPUNIT_ASSERT( mask1->empty() );
	CPPUNIT_ASSERT( mask2->empty() );
}

void Test_Bitmask::xor_check()
{
	unsigned int i = 0;
	
	// Anwendung
	(*mask2) = (*mask0) ^ (*mask1);
	(*mask1) ^= (*mask0);
	
	for( ; i < 4; i++ )
	{
		CPPUNIT_ASSERT( mask1->test(bits[i]) );
		CPPUNIT_ASSERT( mask2->test(bits[i]) );

		mask1->reset(bits[i]);
		mask2->reset(bits[i]);
	}	
	for( ; i < 6; i++ )
	{
		CPPUNIT_ASSERT( !mask1->test(bits[i]) );
		CPPUNIT_ASSERT( !mask2->test(bits[i]) );
	}	
	for( ; i < 10; i++ )
	{
		CPPUNIT_ASSERT( mask1->test(bits[i]) );
		CPPUNIT_ASSERT( mask2->test(bits[i]) );

		mask1->reset(bits[i]);
		mask2->reset(bits[i]);
	}

	CPPUNIT_ASSERT( mask1->empty() );
	CPPUNIT_ASSERT( mask2->empty() );
}

void Test_Bitmask::diff_check()
{
	unsigned int i = 0;
	
	// Anwendung
	(*mask2) = (*mask0) - (*mask1);
	(*mask1) -= (*mask0);
	
	for( ; i < 4; i++ )
	{
		CPPUNIT_ASSERT( !mask1->test(bits[i]) );
		CPPUNIT_ASSERT( mask2->test(bits[i]) );

		mask2->reset(bits[i]);
	}	
	for( ; i < 6; i++ )
	{
		CPPUNIT_ASSERT( !mask1->test(bits[i]) );
		CPPUNIT_ASSERT( !mask2->test(bits[i]) );
	}	
	for( ; i < 10; i++ )
	{
		CPPUNIT_ASSERT( mask1->test(bits[i]) );
		CPPUNIT_ASSERT( !mask2->test(bits[i]) );

		mask1->reset(bits[i]);
	}

	CPPUNIT_ASSERT( mask1->empty() );
	CPPUNIT_ASSERT( mask2->empty() );
}

void Test_Bitmask::assignment_check()
{
	mask2 = new Bitmask(*mask0);
	(*mask3) = (*mask0);

	mask0->clear();

	// Korrekte, tiefe Kopie
	for(unsigned int i=0; i<6; i++)
	{
		CPPUNIT_ASSERT( mask2->test(bits[i]) );
		CPPUNIT_ASSERT( mask3->test(bits[i]) );

		mask2->reset(bits[i]);
		mask3->reset(bits[i]);
	}

	CPPUNIT_ASSERT( mask2->empty() );
	CPPUNIT_ASSERT( mask3->empty() );
}

void Test_Bitmask::relation_check()
{
	// leere Masken gleich, aber keine gemeinsamen Bits
	CPPUNIT_ASSERT( !((*mask2) <= (*mask2)) && !((*mask2) <= (*mask2)) );
	CPPUNIT_ASSERT( ((*mask2) == (*mask2)) && ((*mask2) == (*mask2)) );

	// identische, nichtleere Masken gleich und gemeinsame Bits
	CPPUNIT_ASSERT( ((*mask0) <= (*mask0)) && ((*mask0) <= (*mask0)) );
	CPPUNIT_ASSERT( ((*mask0) == (*mask0)) && ((*mask0) == (*mask0)) );
	
	// ueberlappende, nichtleere Masken ungleich aber gemeinsame Bits
	CPPUNIT_ASSERT( ((*mask0) <= (*mask1)) && ((*mask1) <= (*mask0)) );
	CPPUNIT_ASSERT( !((*mask0) == (*mask1)) && !((*mask1) == (*mask0)) );

	// disjunkte, nichtleere Masken ohne gemeinsame Bits
	CPPUNIT_ASSERT( !(((*mask0)^(*mask0)) <= ((*mask0)^(*mask0))) );
}

