#include <xmlsequencer.h>
#include <macros.h>

#include <utility/xmlhelper.h>

#include <iostream>
#include <string>
#include <cassert>

using namespace std;

void XmlSequencer::initVarDef (DOMElement * vardef, Sequencer::Parameter* pptr)
{
	pptr->value = parseAttribute(vardef, "startValue");
	pptr->increment = parseAttribute(vardef, "addValue");
	pptr->name = (string)parseAttribute(vardef, "cinsimVar");
}


void XmlSequencer::initVarDefs (std::vector<DOMElement*> vardefs)
{
	_parameters.resize (vardefs.size());

	for (unsigned int i = 0; i < vardefs.size (); i++)
		initVarDef (vardefs[i], &_parameters[i]);
}


void XmlSequencer::initSimulation (DOMElement* net)
{
	assert(net);
	
	DOMElement* globals = getFirstChildByName(net, "simsettings"); 
	
	if(!globals)
		_ERROR("Failed to locate global settings!", 1);
	
	_setup.stype = (std::string)parseAttribute(globals, "simulationType");
	_setup.sname = (std::string)parseAttribute(globals, "title");
	_setup.tstart = parseAttribute(globals, "analysisStart");
	_setup.tend = _setup.tstart + 
	    (uint)parseAttribute(globals, "analysisLength") - 1;
	_setup.nofsim = parseAttribute(globals, "numberOfSimulations");
	_setup.nofobs = getChildElementsByName(net, "measurevar").size();
	
	if(!_setup.nofobs)
		_ERROR("Failed to determine the number of observations!", 1);
		
	if(_setup.stype == "Terminating")
	{
		_setup.nofobs *= _setup.tend-_setup.tstart+1;
	}
	
	return;
}

void XmlSequencer::init ()
{
	// XML-Element mit globalen Einstellungen und Variablen
	DOMElement *net = getFirstChildByName(_document->getDocumentElement(), "net");
		
	if(!net)
		_ERROR("Wrong type of network description at top level!", 1);
	
	// Parameter der Sequenz initialisieren
	initSimulation ( net );
		
	// Variablen initialisieren
	initVarDefs ( getChildElementsByName(net, "cinsimvar") );
}
